@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -54,7 +54,7 @@ auth_cram_md5_options_block auth_cram_md5_option_defaults = {
 #  ifdef MACRO_PREDEF
 
 /* Dummy values */
-void auth_cram_md5_init(auth_instance *ablock) {}
+void auth_cram_md5_init(driver_instance* ablock) {}
 int auth_cram_md5_server(auth_instance *ablock, uschar *data) {return 0;}
 int auth_cram_md5_client(auth_instance *ablock, void *sx, int timeout,
     uschar *buffer, int buffsize) {return 0;}
@@ -71,15 +71,18 @@ enable consistency checks to be done, or anything else that needs
 to be set up. */
 
 void
-auth_cram_md5_init(auth_instance *ablock)
+auth_cram_md5_init(driver_instance * a)
 {
-auth_cram_md5_options_block *ob =
-  (auth_cram_md5_options_block *)(ablock->options_block);
-if (ob->server_secret != NULL) ablock->server = TRUE;
-if (ob->client_secret != NULL)
+auth_instance * ablock = (auth_instance *)a;
+auth_cram_md5_options_block * ob =
+  (auth_cram_md5_options_block *)(a->options_block);
+
+if (ob->server_secret)
+  ablock->server = TRUE;
+if (ob->client_secret)
   {
   ablock->client = TRUE;
-  if (ob->client_name == NULL) ob->client_name = primary_hostname;
+  if (!ob->client_name) ob->client_name = primary_hostname;
   }
 }
 
@@ -169,8 +172,7 @@ md5_end(&base, md5secret, 16, digestptr);
 int
 auth_cram_md5_server(auth_instance * ablock, uschar * data)
 {
-auth_cram_md5_options_block * ob =
-  (auth_cram_md5_options_block *)(ablock->options_block);
+auth_cram_md5_options_block * ob = ablock->drinst.options_block;
 uschar * challenge = string_sprintf("<%d.%ld@%s>", getpid(),
     (long int) time(NULL), primary_hostname);
 uschar * clear, * secret;
@@ -228,13 +230,10 @@ compute_cram_md5(secret, challenge, digest);
 
 HDEBUG(D_auth)
   {
-  uschar buff[64];
   debug_printf("CRAM-MD5: user name = %s\n", auth_vars[0]);
   debug_printf("          challenge = %s\n", challenge);
   debug_printf("          received  = %s\n", clear);
-  Ustrcpy(buff, US"          digest    = ");
-  for (i = 0; i < 16; i++) sprintf(CS buff+22+2*i, "%02x", digest[i]);
-  debug_printf("%.54s\n", buff);
+  debug_printf("          digest    = %.16H\n", digest);
   }
 
 /* We now have to compare the digest, which is 16 bytes in binary, with the
@@ -269,8 +268,7 @@ auth_cram_md5_client(
   uschar *buffer,                        /* for reading response */
   int buffsize)                          /* size of buffer */
 {
-auth_cram_md5_options_block *ob =
-  (auth_cram_md5_options_block *)(ablock->options_block);
+auth_cram_md5_options_block * ob = ablock->drinst.options_block;
 uschar *secret = expand_string(ob->client_secret);
 uschar *name = expand_string(ob->client_name);
 uschar *challenge, *p;
@@ -287,10 +285,10 @@ if (!secret || !name)
     *buffer = 0;           /* No message */
     return CANCELLED;
     }
-  string_format(buffer, buffsize, "expansion of \"%s\" failed in "
+  string_format(buffer, buffsize, "expansion of %q failed in "
     "%s authenticator: %s",
     !secret ? ob->client_secret : ob->client_name,
-    ablock->name, expand_string_message);
+    ablock->drinst.name, expand_string_message);
   return ERROR;
   }
 
@@ -313,29 +311,50 @@ if (b64decode(buffer + 4, &challenge, buffer + 4) < 0)
 
 compute_cram_md5(secret, challenge, digest);
 
-/* Create the response from the user name plus the CRAM-MD5 digest */
-
-string_format(big_buffer, big_buffer_size - 36, "%s", name);
-for (p = big_buffer; *p; ) p++;
-*p++ = ' ';
-
-for (i = 0; i < 16; i++)
-  p += sprintf(CS p, "%02x", digest[i]);
-
-/* Send the response, in base 64, and check the result. The response is
-in big_buffer, but b64encode() returns its result in working store,
-so calling smtp_write_command(), which uses big_buffer, is OK. */
-
-buffer[0] = 0;
-if (smtp_write_command(sx, SCMD_FLUSH, "%s\r\n", b64encode(CUS big_buffer,
-  p - big_buffer)) < 0) return FAIL_SEND;
+/* Create the response from the user name plus the CRAM-MD5 digest.
+Send it, in base 64, and check the result. */
+  {
+  int len;
+  p = string_sprintf("%s %.16H%n", name, digest, &len);
+  if (smtp_write_command(sx, SCMD_FLUSH, "%s\r\n", b64encode(p, len)) < 0)
+    return FAIL_SEND;
+  }
 
 return smtp_read_response(sx, US buffer, buffsize, '2', timeout)
   ? OK : FAIL;
 }
+
+
+# ifdef DYNLOOKUP
+#  define cram_md5_auth_info _auth_info
+# endif
 #  endif  /*AUTH_CRAM_MD5*/
 # endif  /*!STAND_ALONE*/
+#endif	/*!MACRO_PREDEF*/
 
+#ifndef STAND_ALONE
+# ifdef AUTH_CRAM_MD5
+
+auth_info cram_md5_auth_info = {
+.drinfo = {
+  .driver_name =	US"cram_md5",                   /* lookup name */
+  .options =		auth_cram_md5_options,
+  .options_count =	&auth_cram_md5_options_count,
+  .options_block =	&auth_cram_md5_option_defaults,
+  .options_len =	sizeof(auth_cram_md5_options_block),
+  .init =		auth_cram_md5_init,
+# ifdef DYNLOOKUP
+  .dyn_magic =		AUTH_MAGIC,
+# endif
+  },
+.servercode =		auth_cram_md5_server,
+.clientcode =		auth_cram_md5_client,
+.version_report =	NULL,
+.macros_create =	NULL,
+};
+
+# endif  /*AUTH_CRAM_MD5*/
+#endif  /*!STAND_ALONE*/
 
 /*************************************************
 **************************************************
@@ -362,5 +381,4 @@ return 0;
 
 # endif	/*STAND_ALONE*/
 
-#endif	/*!MACRO_PREDEF*/
 /* End of cram_md5.c */
