@@ -2,6 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -32,7 +33,7 @@ extern auth_spa_options_block auth_spa_option_defaults;
 
 /* The entry points for the mechanism */
 
-extern void auth_spa_init(auth_instance *);
+extern void auth_spa_init(driver_instance *);
 extern int auth_spa_server(auth_instance *, uschar *);
 extern int auth_spa_client(auth_instance *, void *, int, uschar *, int);
 
