@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2024 */
+/* Copyright (c) The Exim Maintainers 2024 - 2025 */
 /* Copyright (c) Jeremy Harris 1995 - 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -47,7 +47,7 @@ auth_tls_options_block auth_tls_option_defaults = {
 #ifdef MACRO_PREDEF
 
 /* Dummy values */
-void auth_tls_init(auth_instance *ablock) {}
+void auth_tls_init(driver_instance *ablock) {}
 int auth_tls_server(auth_instance *ablock, uschar *data) {return 0;}
 int auth_tls_client(auth_instance *ablock, void * sx,
   int timeout, uschar *buffer, int buffsize) {return 0;}
@@ -66,9 +66,10 @@ enable consistency checks to be done, or anything else that needs
 to be set up. */
 
 void
-auth_tls_init(auth_instance *ablock)
+auth_tls_init(driver_instance * a)
 {
-ablock->public_name = ablock->name;	/* needed for core code */
+auth_instance * ablock = (auth_instance *)a;
+ablock->public_name = a->name;	/* needed for core code */
 }
 
 
@@ -82,7 +83,7 @@ ablock->public_name = ablock->name;	/* needed for core code */
 int
 auth_tls_server(auth_instance *ablock, uschar *data)
 {
-auth_tls_options_block * ob = (auth_tls_options_block *)ablock->options_block;
+auth_tls_options_block * ob = ablock->drinst.options_block;
 
 if (ob->server_param1)
   auth_vars[expand_nmax++] = expand_string(ob->server_param1);
@@ -94,6 +95,28 @@ return auth_check_serv_cond(ablock);
 }
 
 
+# ifdef DYNLOOKUP
+#  define tls_auth_info _auth_info
+# endif
 #endif	/*!MACRO_PREDEF*/
+
+auth_info tls_auth_info = {
+.drinfo = {
+  .driver_name =	US"tls",                   /* lookup name */
+  .options =		auth_tls_options,
+  .options_count =	&auth_tls_options_count,
+  .options_block =	&auth_tls_option_defaults,
+  .options_len =	sizeof(auth_tls_options_block),
+  .init =		auth_tls_init,
+# ifdef DYNLOOKUP
+  .dyn_magic =		AUTH_MAGIC,
+# endif
+  },
+.servercode =		auth_tls_server,
+.clientcode =		NULL,
+.version_report =	NULL,
+.macros_create =	NULL,
+};
+
 #endif	/*AUTH_TLS*/
 /* End of tls.c */
