@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -31,6 +31,7 @@ characters. */
 
 
 #include "exim.h"
+#include "hintsdb.h"
 
 uschar * spool_directory = NULL;	/* dummy for hintsdb.h */
 
@@ -42,6 +43,8 @@ millisleep(int msec)
 uschar *
 readconf_printtime(int t)
 { return NULL; }
+const uschar * expand_string_2(const uschar * string, BOOL * textonly_p)
+{return NULL; }
 void *
 store_get_3(int size, const void * proto_mem, const char *filename, int linenumber)
 { return NULL; }
@@ -88,6 +91,27 @@ if it is made static. */
 const uschar *hex_digits = CUS"0123456789abcdef";
 
 
+/*******************
+*   Debug output   *
+*******************/
+
+unsigned int debug_selector = 0;	/* set -1 for debugging */
+
+void
+debug_printf(const char * fmt, ...)
+{
+va_list ap;
+va_start(ap, fmt); vfprintf(stderr, fmt, ap); va_end(ap);
+}
+void
+debug_printf_indent(const char * fmt, ...)
+{
+va_list ap;
+va_start(ap, fmt); vfprintf(stderr, fmt, ap); va_end(ap);
+}
+
+
+
 #ifdef STRERROR_FROM_ERRLIST
 /* Some old-fashioned systems still around (e.g. SunOS4) don't have strerror()
 in their libraries, but can provide the same facility by this simple
@@ -316,8 +340,8 @@ while (Ufgets(line, max_insize, f) != NULL)
   else
     {
     int i, rc;
-    uschar *s = line;
-    uschar *keystart;
+    uschar * s = line;
+    const uschar * keystart;
 
     if (started)
       {
@@ -357,16 +381,16 @@ while (Ufgets(line, max_insize, f) != NULL)
 
     if (*s == '\"')
       {
-      uschar *t = s++;
+      uschar * t = s++;
       keystart = t;
-      while (*s != 0 && *s != '\"')
+      while (*s && *s != '\"')
         {
 	*t++ = *s == '\\'
 	? string_interpret_escape((const uschar **)&s)
 	: *s;
         s++;
         }
-      if (*s != 0) s++;               /* Past terminating " */
+      if (*s) s++;               /* Past terminating " */
       exim_datum_size_set(&key, t - keystart + add_zero);
       }
     else
@@ -400,7 +424,7 @@ while (Ufgets(line, max_insize, f) != NULL)
       s++;
       while (isspace(*s)) s++;
       }
-    if (*s != 0)
+    if (*s)
       {
       Ustrcpy(bptr, s);
       bptr += p - s;
