@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -22,7 +22,7 @@ optionlist optionlist_auths[] = {
   { "client_set_id", opt_stringptr | opt_public,
                  OPT_OFF(auth_instance, set_client_id) },
   { "driver",        opt_stringptr | opt_public,
-                 OPT_OFF(auth_instance, driver_name) },
+                 OPT_OFF(auth_instance, drinst.driver_name) },
   { "public_name",   opt_stringptr | opt_public,
                  OPT_OFF(auth_instance, public_name) },
   { "server_advertise_condition", opt_stringptr | opt_public,
@@ -99,11 +99,12 @@ int     sqlite_lock_timeout    = 5;
 BOOL    move_frozen_messages   = FALSE;
 #endif
 
-/* These variables are outside the #ifdef because it keeps the code less
+/* These variables are outside the TLS #ifdef because it keeps the code less
 cluttered in several places (e.g. during logging) if we can always refer to
 them. Also, the tls_ variables are now always visible.  Note that these are
 only used for smtp connections, not for service-daemon access. */
 
+#ifndef MACRO_PREDEF
 tls_support tls_in = {
  .active =		{.sock = -1}
  /* all other elements zero */
@@ -112,6 +113,7 @@ tls_support tls_out = {
  .active =		{.sock = -1},
  /* all other elements zero */
 };
+#endif
 
 uschar *dsn_envid              = NULL;
 int     dsn_ret                = 0;
@@ -133,10 +135,12 @@ that's the interop problem which has been observed: GnuTLS suggesting a higher
 bit-count as "NORMAL" (2432) and Thunderbird dropping connection. */
 int     tls_dh_max_bits        = 2236;
 uschar *tls_dhparam            = NULL;
+uschar *tls_early_banner_hosts = US"";
 uschar *tls_eccurve            = US"auto";
 # ifndef DISABLE_OCSP
 uschar *tls_ocsp_file          = NULL;
 # endif
+uschar *tls_on_connect_ports   = NULL;
 uschar *tls_privatekey         = NULL;
 BOOL    tls_remember_esmtp     = FALSE;
 uschar *tls_require_ciphers    = NULL;
@@ -174,7 +178,7 @@ int	(*lwr_receive_ungetc)(int)	= stdin_ungetc;
 BOOL	(*lwr_receive_hasc)(void)	= stdin_hasc;
 
 int	(*receive_getc)(unsigned) 	= stdin_getc;
-uschar * (*receive_getbuf)(unsigned *) 	= NULL;
+uschar * (*receive_getbuf)(unsigned *) 	= NULL;		/* can return NULL */
 void	(*receive_get_cache)(unsigned)	= NULL;
 BOOL	(*receive_hasc)(void)		= stdin_hasc;
 int	(*receive_ungetc)(int)    	= stdin_ungetc;
@@ -207,7 +211,7 @@ const uschar **address_expansions[ADDRESS_EXPANSIONS_COUNT] = {
   CUSS &self_hostname,
   NULL };
 
-int address_expansions_count = sizeof(address_expansions)/sizeof(uschar **);
+int address_expansions_count = nelem(address_expansions);
 
 /******************************************************************************/
 /* General global variables.  Boolean flags are done as a group
@@ -411,9 +415,6 @@ BOOL    smtp_enforce_sync      = TRUE;
 BOOL    smtp_etrn_serialize    = TRUE;
 BOOL    smtp_input             = FALSE;
 BOOL    smtp_return_error_details = FALSE;
-#ifdef SUPPORT_SPF
-BOOL    spf_result_guessed     = FALSE;
-#endif
 BOOL    split_spool_directory  = FALSE;
 BOOL    spool_wireformat       = FALSE;
 BOOL    strict_acl_vars        = FALSE;
@@ -444,6 +445,7 @@ uschar *acl_not_smtp_mime      = NULL;
 #endif
 uschar *acl_not_smtp_start     = NULL;
 uschar *acl_removed_headers    = NULL;
+uschar *acl_smtp_atrn          = NULL;
 uschar *acl_smtp_auth          = NULL;
 uschar *acl_smtp_connect       = NULL;
 uschar *acl_smtp_data          = NULL;
@@ -475,6 +477,7 @@ tree_node *acl_var_c           = NULL;
 tree_node *acl_var_m           = NULL;
 uschar *acl_verify_message     = NULL;
 string_item *acl_warn_logged   = NULL;
+int acl_where		       = ACL_WHERE_UNKNOWN;
 
 /* Names of SMTP places for use in ACL error messages, and corresponding SMTP
 error codes (only those used) */
@@ -489,6 +492,7 @@ uschar *acl_wherenames[]       = { [ACL_WHERE_RCPT] =		US"RCPT",
                                    [ACL_WHERE_PRDR] =		US"PRDR",
 #endif
                                    [ACL_WHERE_NOTSMTP] =	US"non-SMTP",
+                                   [ACL_WHERE_ATRN] =		US"ATRN",
                                    [ACL_WHERE_AUTH] =		US"AUTH",
                                    [ACL_WHERE_CONNECT] =	US"connection",
                                    [ACL_WHERE_ETRN] =		US"ETRN",
@@ -516,6 +520,7 @@ uschar *acl_wherecodes[]       = { [ACL_WHERE_RCPT] =	US"550",
 #ifndef DISABLE_PRDR
                                    [ACL_WHERE_PRDR] =	US"550",
 #endif
+                                   [ACL_WHERE_ATRN] =	US"450",
                                    [ACL_WHERE_AUTH] =	US"503",
                                    [ACL_WHERE_CONNECT] = US"550",
                                    [ACL_WHERE_ETRN] =	US"458",
@@ -531,85 +536,14 @@ uschar *acl_wherecodes[]       = { [ACL_WHERE_RCPT] =	US"550",
 uschar *add_environment        = NULL;
 address_item  *addr_duplicate  = NULL;
 
-address_item address_defaults = {
-  .next =		NULL,
-  .parent =		NULL,
-  .first =		NULL,
-  .dupof =		NULL,
-  .start_router =	NULL,
-  .router =		NULL,
-  .transport =		NULL,
-  .host_list =		NULL,
-  .host_used =		NULL,
-  .fallback_hosts =	NULL,
-  .reply =		NULL,
-  .retries =		NULL,
-  .address =		NULL,
-  .unique =		NULL,
-  .cc_local_part =	NULL,
-  .lc_local_part =	NULL,
-  .local_part =		NULL,
-  .prefix =		NULL,
-  .prefix_v =		NULL,
-  .suffix =		NULL,
-  .suffix_v =		NULL,
-  .domain =		NULL,
-  .address_retry_key =	NULL,
-  .domain_retry_key =	NULL,
-  .current_dir =	NULL,
-  .home_dir =		NULL,
-  .message =		NULL,
-  .user_message =	NULL,
-  .onetime_parent =	NULL,
-  .pipe_expandn =	NULL,
-  .return_filename =	NULL,
-  .self_hostname =	NULL,
-  .shadow_message =	NULL,
-#ifndef DISABLE_TLS
-  .cipher =		NULL,
-  .ourcert =		NULL,
-  .peercert =		NULL,
-  .peerdn =		NULL,
-  .ocsp =		OCSP_NOT_REQ,
-#endif
-#ifdef EXPERIMENTAL_DSN_INFO
-  .smtp_greeting =	NULL,
-  .helo_response =	NULL,
-#endif
-  .authenticator =	NULL,
-  .auth_id =		NULL,
-  .auth_sndr =		NULL,
-  .dsn_orcpt =		NULL,
-  .dsn_flags =		0,
-  .dsn_aware =		0,
+address_item address_defaults = {    /* All unmentioned elements 0/NULL/FALSE */
   .uid =		(uid_t)(-1),
   .gid =		(gid_t)(-1),
-  .flags =		{ 0 },
-  .domain_cache =	{ 0 },                /* domain_cache - any larger array should be zeroed */
-  .localpart_cache =	{ 0 },                /* localpart_cache - ditto */
   .mode =		-1,
-  .more_errno =		0,
-  .delivery_time =	{.tv_sec = 0, .tv_usec = 0},
   .basic_errno =	ERRNO_UNKNOWNERROR,
-  .child_count =	0,
   .return_file =	-1,
   .special_action =	SPECIAL_NONE,
   .transport_return =	DEFER,
-  .prop = {					/* fields that are propagated to children */
-    .address_data =	NULL,
-    .domain_data =	NULL,
-    .localpart_data =	NULL,
-    .errors_address =	NULL,
-    .extra_headers =	NULL,
-    .remove_headers =	NULL,
-    .variables =	NULL,
-    .ignore_error =	FALSE,
-#ifdef SUPPORT_I18N
-    .utf8_msg =		FALSE,
-    .utf8_downcvt =	FALSE,
-    .utf8_downcvt_maybe = FALSE
-#endif
-  }
 };
 
 const uschar *address_file           = NULL;
@@ -618,36 +552,17 @@ tree_node *addresslist_anchor  = NULL;
 int     addresslist_count      = 0;
 gid_t  *admin_groups           = NULL;
 
-#ifdef EXPERIMENTAL_ARC
-struct arc_set *arc_received	= NULL;
-int     arc_received_instance	= 0;
-int     arc_oldest_pass		= 0;
-const uschar *arc_state		= NULL;
-const uschar *arc_state_reason	= NULL;
-#endif
+const uschar *atrn_domains     = NULL;
+const uschar *atrn_host        = NULL;
+const uschar *atrn_mode        = NULL;
 
 uschar *authenticated_fail_id  = NULL;
 uschar *authenticated_id       = NULL;
 uschar *authenticated_sender   = NULL;
 auth_instance  *auths          = NULL;
 uschar *auth_advertise_hosts   = US"*";
-auth_instance auth_defaults    = {
-    .next =		NULL,
-    .name =		NULL,
-    .info =		NULL,
-    .options_block =	NULL,
-    .driver_name =	NULL,
-    .advertise_condition = NULL,
-    .client_condition =	NULL,
-    .public_name =	NULL,
-    .set_id =		NULL,
-    .set_client_id =	NULL,
-    .mail_auth_condition = NULL,
-    .server_debug_string = NULL,
-    .server_condition =	NULL,
-    .client =		FALSE,
-    .server =		FALSE,
-    .advertised =	FALSE
+auth_instance auth_defaults    = {   /* All unmentioned elements 0/NULL/FALSE */
+    0
 };
 
 uschar *auth_defer_msg         = US"reason not recorded";
@@ -722,7 +637,7 @@ int     clmacro_count          = 0;
 uschar *clmacros[MAX_CLMACROS];
 FILE   *config_file            = NULL;
 const uschar *config_filename  = NULL;
-int     config_lineno          = 0;
+unsigned config_lineno         = 0;
 #ifdef CONFIGURE_GROUP
 gid_t   config_gid             = CONFIGURE_GROUP;
 #else
@@ -739,25 +654,34 @@ uid_t   config_uid             = CONFIGURE_OWNER;
 uid_t   config_uid             = 0;
 #endif
 
-uint64_t connection_id	       = 0L;
+const uschar *connection_id    = NULL;
 int     connection_max_messages= -1;
-uschar *continue_proxy_cipher  = NULL;
-BOOL    continue_proxy_dane    = FALSE;
-uschar *continue_proxy_sni     = NULL;
-uschar *continue_hostname      = NULL;
-uschar *continue_host_address  = NULL;
-int     continue_sequence      = 1;
-uschar *continue_transport     = NULL;
+unsigned continue_flags	       = 0;
 #ifndef DISABLE_ESMTP_LIMITS
 unsigned continue_limit_mail   = 0;
 unsigned continue_limit_rcpt   = 0;
 unsigned continue_limit_rcptdom= 0;
+int	continue_fd	       = -1;
+uschar *continue_proxy_cipher  = NULL;
+BOOL    continue_proxy_dane    = FALSE;
+uschar *continue_proxy_sni     = NULL;
+const uschar *continue_hostname      = NULL;
+const uschar *continue_host_address  = NULL;
+int     continue_host_port     = 0;
+uschar  continue_next_id[MESSAGE_ID_LENGTH +1] = {[0]='\0'};
+unsigned continue_sequence     = 1;
+const uschar *continue_transport = NULL;
+#ifndef COMPILE_UTILITY
+open_db *continue_retry_db     = NULL;
+open_db *continue_wait_db      = NULL;
+#endif
 #endif
 
 uschar *csa_status             = NULL;
 cut_t   cutthrough = {
   .callout_hold_only =	FALSE,				/* verify-only: normal delivery */
   .delivery =		FALSE,				/* when to attempt */
+  .tpt_sender =		FALSE,				/* use tpt's sender */
   .defer_pass =		FALSE,				/* on defer: spool locally */
   .is_tls =		FALSE,				/* not a TLS conn yet */
   .cctx =		{.sock = -1},			/* open connection */
@@ -784,7 +708,7 @@ int     debug_notall[]         = {
   -1
 };
 bit_table debug_options[]      = { /* must be in alphabetical order and use
-				 only the enum values from macro.h */
+				 only the enum values from macros.h */
   BIT_TABLE(D, acl),
   BIT_TABLE(D, all),
   BIT_TABLE(D, auth),
@@ -823,6 +747,7 @@ uschar   debuglog_name[LOG_NAME_SIZE] = {0};
 unsigned debug_pretrigger_bsize	= 0;
 uschar * debug_pretrigger_buf	= NULL;
 unsigned int debug_selector	= 0;
+BOOL	 debug_startup		= FALSE;
 
 int     delay_warning[DELAY_WARNING_SIZE] = { DELAY_WARNING_SIZE, 1, 24*60*60 };
 uschar *delay_warning_condition=
@@ -858,34 +783,6 @@ address_item  *deliver_recipients = NULL;
 uschar *deliver_selectstring   = NULL;
 uschar *deliver_selectstring_sender = NULL;
 
-#ifndef DISABLE_DKIM
-unsigned dkim_collect_input      = 0;
-uschar *dkim_cur_signer          = NULL;
-int     dkim_key_length          = 0;
-void   *dkim_signatures		 = NULL;
-uschar *dkim_signers             = NULL;
-uschar *dkim_signing_domain      = NULL;
-uschar *dkim_signing_selector    = NULL;
-gstring *dkim_signing_record	 = NULL;
-uschar *dkim_verify_hashes       = US"sha256:sha512";
-uschar *dkim_verify_keytypes     = US"ed25519:rsa";
-uschar *dkim_verify_min_keysizes = US"rsa=1024 ed25519=250";
-BOOL	dkim_verify_minimal      = FALSE;
-uschar *dkim_verify_overall      = NULL;
-uschar *dkim_verify_signers      = US"$dkim_signers";
-uschar *dkim_verify_status	 = NULL;
-uschar *dkim_verify_reason	 = NULL;
-#endif
-#ifdef SUPPORT_DMARC
-uschar *dmarc_domain_policy     = NULL;
-uschar *dmarc_forensic_sender   = NULL;
-uschar *dmarc_history_file      = NULL;
-uschar *dmarc_status            = NULL;
-uschar *dmarc_status_text       = NULL;
-uschar *dmarc_tld_file          = NULL;
-uschar *dmarc_used_domain       = NULL;
-#endif
-
 uschar *dns_again_means_nonexist = NULL;
 int     dns_csa_search_limit   = 5;
 int	dns_cname_loops	       = 1;
@@ -1146,6 +1043,7 @@ bit_table log_options[]        = { /* must be in alphabetical order,
   BIT_TABLE(L, subject),
   BIT_TABLE(L, tls_certificate_verified),
   BIT_TABLE(L, tls_cipher),
+  BIT_TABLE(L, tls_on_connect),
   BIT_TABLE(L, tls_peerdn),
   BIT_TABLE(L, tls_resumption),
   BIT_TABLE(L, tls_sni),
@@ -1153,6 +1051,7 @@ bit_table log_options[]        = { /* must be in alphabetical order,
 };
 int     log_options_count      = nelem(log_options);
 
+const uschar *log_ports	       = NULL;
 int     log_reject_target      = 0;
 unsigned int log_selector[log_selector_size]; /* initialized in main() */
 uschar *log_selector_string    = NULL;
@@ -1246,9 +1145,9 @@ const uschar *process_purpose  = US"fresh-exec";
 
 #if defined(SUPPORT_PROXY) || defined(SUPPORT_SOCKS) || defined(EXPERIMENTAL_XCLIENT)
 uschar *proxy_external_address = NULL;
-int     proxy_external_port    = 0;
+unsigned proxy_external_port   = 0;
 uschar *proxy_local_address    = NULL;
-int     proxy_local_port       = 0;
+unsigned proxy_local_port      = 0;
 int     proxy_protocol_timeout = 3;
 #endif
 
@@ -1360,88 +1259,28 @@ uid_t   root_gid               = ROOT_GID;
 uid_t   root_uid               = ROOT_UID;
 
 router_instance  *routers  = NULL;
-router_instance  router_defaults = {
-    .next =			NULL,
-    .name =			NULL,
-    .info =			NULL,
-    .options_block =		NULL,
-    .driver_name =		NULL,
 
-    .address_data =		NULL,
-#ifdef EXPERIMENTAL_BRIGHTMAIL
-    .bmi_rule =			NULL,
-#endif
-    .cannot_route_message =	NULL,
-    .condition =		NULL,
-    .current_directory =	NULL,
-    .debug_string =		NULL,
-    .domains =			NULL,
-    .errors_to =		NULL,
-    .expand_gid =		NULL,
-    .expand_uid =		NULL,
-    .expand_more =		NULL,
-    .expand_unseen =		NULL,
-    .extra_headers =		NULL,
-    .fallback_hosts =		NULL,
-    .home_directory =		NULL,
-    .ignore_target_hosts =	NULL,
-    .local_parts =		NULL,
-    .pass_router_name =		NULL,
-    .prefix =			NULL,
-    .redirect_router_name =	NULL,
-    .remove_headers =		NULL,
-    .require_files =		NULL,
-    .router_home_directory =	NULL,
+/* All elements not mentioned will be 0/NULL/FALSE */
+router_instance  router_defaults = {
     .self =			US"freeze",
-    .senders =			NULL,
-    .suffix =			NULL,
-    .translate_ip_address =	NULL,
-    .transport_name =		NULL,
 
     .address_test =		TRUE,
-#ifdef EXPERIMENTAL_BRIGHTMAIL
-    .bmi_deliver_alternate =	FALSE,
-    .bmi_deliver_default =	FALSE,
-    .bmi_dont_deliver =		FALSE,
-#endif
     .expn =			TRUE,
-    .caseful_local_part =	FALSE,
-    .check_local_user =		FALSE,
-    .disable_logging =		FALSE,
-    .fail_verify_recipient =	FALSE,
-    .fail_verify_sender =	FALSE,
-    .gid_set =			FALSE,
-    .initgroups =		FALSE,
     .log_as_local =		TRUE_UNSET,
     .more =			TRUE,
-    .pass_on_timeout =		FALSE,
-    .prefix_optional =		FALSE,
     .repeat_use =		TRUE,
     .retry_use_local_part =	TRUE_UNSET,
-    .same_domain_copy_routing =	FALSE,
-    .self_rewrite =		FALSE,
-    .set =			NULL,
-    .suffix_optional =		FALSE,
-    .verify_only =		FALSE,
     .verify_recipient =		TRUE,
     .verify_sender =		TRUE,
-    .uid_set =			FALSE,
-    .unseen =			FALSE,
-    .dsn_lasthop =		FALSE,
 
     .self_code =		self_freeze,
     .uid =			(uid_t)(-1),
     .gid =			(gid_t)(-1),
 
-    .fallback_hostlist =	NULL,
-    .transport =		NULL,
-    .pass_router =		NULL,
-    .redirect_router =		NULL,
-
     .dnssec =                   { .request= US"*", .require=NULL },
 };
 
-uschar *router_name            = NULL;
+const uschar *router_name      = NULL;
 tree_node *router_var	       = NULL;
 
 ip_address_item *running_interfaces = NULL;
@@ -1513,22 +1352,21 @@ uschar  smtp_connection_had[SMTP_HBUFF_SIZE];
 int     smtp_connect_backlog   = 20;
 double  smtp_delay_mail        = 0.0;
 double  smtp_delay_rcpt        = 0.0;
-FILE   *smtp_in                = NULL;
+int	smtp_in_fd	       = -1;
 int     smtp_listen_backlog    = 0;
 int     smtp_load_reserve      = -1;
 int     smtp_mailcmd_count     = 0;
 int     smtp_mailcmd_max       = -1;
-FILE   *smtp_out               = NULL;
+int	smtp_out_fd	       = -1;
 uschar *smtp_etrn_command      = NULL;
 int     smtp_max_synprot_errors= 3;
 int     smtp_max_unknown_commands = 3;
-uschar *smtp_notquit_reason    = NULL;
+const uschar *smtp_notquit_reason = NULL;
 unsigned smtp_peer_options     = 0;
 unsigned smtp_peer_options_wrap= 0;
 uschar *smtp_ratelimit_hosts   = NULL;
 uschar *smtp_ratelimit_mail    = NULL;
 uschar *smtp_ratelimit_rcpt    = NULL;
-uschar *smtp_read_error        = US"";
 int     smtp_receive_timeout   = 5*60;
 uschar *smtp_receive_timeout_s = NULL;
 uschar *smtp_reserve_hosts     = NULL;
@@ -1552,17 +1390,6 @@ uschar *spam_action            = NULL;
 uschar *spam_score             = NULL;
 uschar *spam_score_int         = NULL;
 #endif
-#ifdef SUPPORT_SPF
-uschar *spf_guess              = US"v=spf1 a/24 mx/24 ptr ?all";
-uschar *spf_header_comment     = NULL;
-uschar *spf_received           = NULL;
-uschar *spf_result             = NULL;
-uschar *spf_smtp_comment       = NULL;
-uschar *spf_smtp_comment_template
-                    /* Used to be: "Please%_see%_http://www.open-spf.org/Why?id=%{S}&ip=%{C}&receiver=%{R}" */
-                               = US"Please%_see%_http://www.open-spf.org/Why";
-
-#endif
 
 FILE   *spool_data_file	       = NULL;
 uschar *spool_directory        = US SPOOL_DIRECTORY
@@ -1589,9 +1416,7 @@ uid_t   system_filter_uid      = (uid_t)-1;
 
 blob	tcp_fastopen_nodata    = { .data = NULL, .len = 0 };
 tfo_state_t tcp_out_fastopen   = TFO_NOT_USED;
-#ifdef USE_TCP_WRAPPERS
-uschar *tcp_wrappers_daemon_name = US TCP_WRAPPERS_DAEMON_NAME;
-#endif
+int	test_harness_identd_port  = IDENT_PORT;
 int     test_harness_load_avg  = 0;
 int     thismessage_size_limit = 0;
 int     timeout_frozen_after   = 0;
@@ -1615,7 +1440,7 @@ transport_instance  transport_defaults = {
 };
 
 int     transport_count;
-uschar *transport_name          = NULL;
+const uschar *transport_name          = NULL;
 int     transport_newlines;
 const uschar **transport_filter_argv  = NULL;
 int     transport_filter_timeout;
@@ -1662,7 +1487,7 @@ uschar *uucp_from_sender       = US"$1";
 uschar *verify_mode	       = NULL;
 uschar *version_copyright      =
  US"Copyright (c) University of Cambridge, 1995 - 2018\n"
-   "(c) The Exim Maintainers and contributors in ACKNOWLEDGMENTS file, 2007 - 2024";
+   "(c) The Exim Maintainers and contributors in ACKNOWLEDGMENTS file, 2007 - 2025";
 uschar *version_date           = US"?";
 uschar *version_cnumber        = US"????";
 uschar *version_string         = US"?";
