@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2023 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2016 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -109,7 +109,7 @@ gs.size = HEADER_ADD_BUFFER_SIZE;
 gs.ptr = 0;
 
 if (!string_vformat(&gs, SVFMT_REBUFFER, format, ap))
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "string too long in header_add: "
+  log_write_die(0, LOG_MAIN, "string too long in header_add: "
     "%.100Y ...", &gs);
 
 if (gs.s != buf) store_release_above(buf);
