@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2023 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -77,7 +77,7 @@ ip_addrinfo(const uschar *address, struct sockaddr_in6 *saddr)
 #ifdef IPV6_USE_INET_PTON
 
   if (inet_pton(AF_INET6, CCS address, &saddr->sin6_addr) != 1)
-    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "unable to parse \"%s\" as an "
+    log_write_die(0, LOG_MAIN, "unable to parse %q as an "
       "IP address", address);
   saddr->sin6_family = AF_INET6;
 
@@ -90,11 +90,11 @@ ip_addrinfo(const uschar *address, struct sockaddr_in6 *saddr)
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_flags = AI_NUMERICHOST;
   if ((rc = getaddrinfo(CCS address, NULL, &hints, &res)) != 0 || res == NULL)
-    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "unable to parse \"%s\" as an "
+    log_write_die(0, LOG_MAIN, "unable to parse %q as an "
       "IP address: %s", address,
-      (rc == 0)? "NULL result returned" : gai_strerror(rc));
-  memcpy(saddr, res->ai_addr, res->ai_addrlen);
-  freeaddrinfo(res);
+      rc == 0 ? "NULL result returned" : gai_strerror(rc));
+  else
+    { memcpy(saddr, res->ai_addr, res->ai_addrlen); freeaddrinfo(res); }
 
 #endif
 }
@@ -157,7 +157,7 @@ Returns:         the result of bind()
 */
 
 int
-ip_bind(int sock, int af, uschar *address, int port)
+ip_bind(int sock, int af, const uschar * address, int port)
 {
 union sockaddr_46 sin;
 int s_len = ip_addr(&sin, af, address, port);
@@ -426,7 +426,7 @@ if (hostname[0] == '[' &&
   uschar * host = string_copyn(hostname+1, namelen-2);
   if (string_is_ip_address(host, NULL) == 0)
     {
-    *errstr = string_sprintf("malformed IP address \"%s\"", hostname);
+    *errstr = string_sprintf("malformed IP address %q", hostname);
     return -1;
     }
   shost.name = shost.address = host;
@@ -649,7 +649,7 @@ Arguments:
   timelimit   the timeout endpoint, seconds-since-epoch
 
 Returns:      > 0 => that much data read
-              <= 0 on error or EOF; errno set - zero for EOF
+              <= 0 on error or EOF; errno set - zero for EOF or zero buffer
 */
 
 int
@@ -657,6 +657,8 @@ ip_recv(client_conn_ctx * cctx, uschar * buffer, int buffsize, time_t timelimit)
 {
 int rc;
 
+if (buffsize == 0)
+  { errno = 0; return 0; }
 if (!fd_ready(cctx->sock, timelimit))
   return -1;
 
