@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -308,8 +308,8 @@ Send fd over socketpair.
 Return: true iff good.
 */
 
-static BOOL
-log_send_fd(const int sock, const int fd)
+BOOL
+send_fd_over_socket(const int sock, const int fd)
 {
 struct msghdr msg;
 union {
@@ -343,8 +343,8 @@ return n == 1;
 Return fd passed over socketpair, or -1 on error.
 */
 
-static int
-log_recv_fd(const int sock)
+int
+recv_fd_from_sock(const int sock)
 {
 struct msghdr msg;
 union {
@@ -415,7 +415,7 @@ else if (euid == root_uid)
          || getgid() != exim_gid || getegid() != exim_gid
 
          || (fd = log_open_already_exim(name)) < 0
-         || !log_send_fd(sock[1], fd)
+         || !send_fd_over_socket(sock[1], fd)
 	 ) _exit(EXIT_FAILURE);
       (void)close(sock[1]);
       _exit(EXIT_SUCCESS);
@@ -424,7 +424,7 @@ else if (euid == root_uid)
     (void)close(sock[1]);
     if (pid > 0)
       {
-      fd = log_recv_fd(sock[0]);
+      fd = recv_fd_from_sock(sock[0]);
       while (waitpid(pid, NULL, 0) == -1 && errno == EINTR);
       }
     (void)close(sock[0]);
@@ -513,7 +513,7 @@ switch (type)
   case lt_debug:
     /* and deal with the debug log (which keeps the datestamp, but does not
     update it) */
-    Ustrcpy(debuglog_name, buffer);
+    sprintf(CS debuglog_name, "%.*s", (int) sizeof(debuglog_name)-1, buffer);
     if (tag)
       {
       if (is_tainted(tag))
@@ -524,7 +524,7 @@ switch (type)
       ok2 = string_format(buffer, sizeof(buffer), "%s%s",
         debuglog_name, tag);
       if (ok2)
-        Ustrcpy(debuglog_name, buffer);
+	sprintf(CS debuglog_name, "%.*s", (int)sizeof(debuglog_name)-1, buffer);
       }
     break;
 
@@ -587,7 +587,7 @@ if (!panic_save_buffer)
   if ((panic_save_buffer = US malloc(LOG_BUFFER_SIZE)))
     memcpy(panic_save_buffer, log_buffer, LOG_BUFFER_SIZE);
 
-log_write(0, LOG_PANIC_DIE, "Cannot open %s log file \"%s\": %s: "
+log_write_die(0, LOG_PANIC_DIE, "Cannot open %s log file %q: %s: "
   "euid=%d egid=%d", log_names[type], buffer, strerror(errno), euid, getegid());
 /* Never returns */
 }
@@ -642,14 +642,40 @@ if (flags & (LOG_CONFIG_FOR & ~LOG_CONFIG))
 else
   {
   if (flags & (LOG_CONFIG_IN & ~LOG_CONFIG))
-    string_fmt_append_noextend(g, " in line %d of %s",
+    if (config_lineno > 0)
+      {
+      if (acl_where != ACL_WHERE_UNKNOWN)
+	string_fmt_append_noextend(g, " in ACL verb at");
+      else
+	string_fmt_append_noextend(g, " in");
+
+      string_fmt_append_noextend(g, " line %d of %s",
 				  config_lineno, config_filename);
+      }
+    else if (router_name)
+      string_fmt_append_noextend(g, " in %s router", router_name);
+    else if (transport_name)
+      string_fmt_append_noextend(g, " in %s transport", transport_name);
 
   string_fmt_append_noextend(g, ":\n  ");
   }
 }
 
 
+/* Log a port number, to be appended to an IP address.
+Filter the operation depending on the log_ports option
+*/
+
+gstring *
+log_portnum(gstring * g, int port)
+{
+return (  !log_ports
+       || match_isinlist(string_sprintf("%d", port),
+		&log_ports, 0, NULL, NULL, MCL_STRING, TRUE, NULL) == OK)
+  ? string_fmt_append(g, ":%d", port) : g;
+}
+
+
 /*************************************************
 *           A write() operation failed           *
 *************************************************/
@@ -681,9 +707,9 @@ if (!panic_save_buffer)
   if ((panic_save_buffer = US malloc(LOG_BUFFER_SIZE)))
     memcpy(panic_save_buffer, log_buffer, LOG_BUFFER_SIZE);
 
-log_write(0, LOG_PANIC_DIE, "failed to write to %s: length=%d result=%d "
+log_write_die(0, LOG_PANIC_DIE, "failed to write to %s: length=%d result=%d "
   "errno=%d (%s)", name, length, rc, save_errno,
-  (save_errno == 0)? "write incomplete" : strerror(save_errno));
+  save_errno == 0 ? "write incomplete" : strerror(save_errno));
 /* Never returns */
 }
 
@@ -734,7 +760,7 @@ return total_written;
 static inline ssize_t
 write_gstring_to_fd_buf(int fd, const gstring * g)
 {
-return write_to_fd_buf(fd, g->s, g->ptr);
+return write_to_fd_buf(fd, g->s, gstring_length(g));
 }
 
 
@@ -827,14 +853,13 @@ Arguments:
 Returns:    nothing
 */
 
-void
-log_write(unsigned int selector, int flags, const char *format, ...)
+static void
+log_vwrite(unsigned int selector, int flags, const char * format, va_list ap)
 {
 int paniclogfd;
 ssize_t written_len;
 gstring gs = { .size = LOG_BUFFER_SIZE-2, .ptr = 0, .s = log_buffer };
 gstring * g = &gs;
-va_list ap;
 
 /* If panic_recurseflag is set, we have failed to open the panic log. This is
 the ultimate disaster. First try to write the message to a debug file and/or
@@ -944,6 +969,7 @@ in one go so that it doesn't get split when multi-processing. */
 
 DEBUG(D_any|D_v)
   {
+  va_list aq;
   string_fmt_append_noextend(g, "LOG:");
 
   /* Show the selector that was passed into the call. */
@@ -967,14 +993,14 @@ DEBUG(D_any|D_v)
   malloc'd.  So use deliberately taint-nonchecking routines to build into
   it, trusting that we will never expand the results. */
 
-  va_start(ap, format);
-  if (!string_vformat(g, SVFMT_TAINT_NOCHK, format, ap))
+  va_copy(aq, ap);
+  if (!string_vformat(g, SVFMT_TAINT_NOCHK, format, aq))
     {
     uschar * s = US"**** log string overflowed log buffer ****";
     gstring_trim(g, Ustrlen(s));
     string_fmt_append_noextend(g, "%s", s);
     }
-  va_end(ap);
+  va_end(aq);
 
   debug_printf("%Y\n", g);
 
@@ -985,8 +1011,7 @@ DEBUG(D_any|D_v)
 /* If no log file is specified, we are in a mess. */
 
 if (!(flags & (LOG_MAIN|LOG_PANIC|LOG_REJECT)))
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "log_write called with no log "
-    "flags set");
+  log_write_die(0, LOG_MAIN, "log_write called with no log flags set");
 
 /* There are some weird circumstances in which logging is disabled. */
 
@@ -1008,9 +1033,9 @@ string_fmt_append_noextend(&gs, "%s ", tod_stamp(tod_log));
 
 if (LOGGING(pid))
   {
-  if (!syslog_pid) pid_position[0] = g->ptr;		/* remember begin … */
+  if (!syslog_pid) pid_position[0] = gstring_length(g);	/* remember begin … */
   string_fmt_append_noextend(g, "[%ld] ", (long)getpid());
-  if (!syslog_pid) pid_position[1] = g->ptr;		/*  … and end+1 of the PID */
+  if (!syslog_pid) pid_position[1] = gstring_length(g);	/*  … and end+1 of the PID */
   }
 
 if (f.really_exim && message_id[0])
@@ -1019,20 +1044,16 @@ if (f.really_exim && message_id[0])
 if (flags & LOG_CONFIG)
   log_config_info(g, flags);
 
-va_start(ap, format);
-  {
-  /* We want to be able to log tainted info, but log_buffer is directly
-  malloc'd.  So use deliberately taint-nonchecking routines to build into
-  it, trusting that we will never expand the results. */
+/* We want to be able to log tainted info, but log_buffer is directly
+malloc'd.  So use deliberately taint-nonchecking routines to build into
+it, trusting that we will never expand the results. */
 
-  if (!string_vformat(g, SVFMT_TAINT_NOCHK, format, ap))
-    {
-    uschar * s = US"**** log string overflowed log buffer ****\n";
-    gstring_trim(g, Ustrlen(s));
-    string_fmt_append_noextend(g, "%s", s);
-    }
+if (!string_vformat(g, SVFMT_TAINT_NOCHK, format, ap))
+  {
+  uschar * s = US"**** log string overflowed log buffer ****";
+  gstring_trim(g, Ustrlen(s));
+  string_fmt_append_noextend(g, "%s", s);
   }
-va_end(ap);
 
 /* Add the raw, unrewritten, sender to the message if required. This is done
 this way because it kind of fits with LOG_RECIPIENTS. */
@@ -1133,9 +1154,10 @@ if (  flags & LOG_MAIN
 
     /* Failing to write to the log is disastrous */
 
-    if ((written_len = write_gstring_to_fd_buf(mainlogfd, g)) != g->ptr)
+    if (  (written_len = write_gstring_to_fd_buf(mainlogfd, g))
+       != gstring_length(g))
       {
-      log_write_failed(US"main log", g->ptr, written_len);
+      log_write_failed(US"main log", gstring_length(g), written_len);
       /* That function does not return */
       }
     }
@@ -1233,7 +1255,8 @@ if (flags & LOG_REJECT)
       if (fstat(rejectlogfd, &statbuf) >= 0) rejectlog_inode = statbuf.st_ino;
       }
 
-    if ((written_len = write_gstring_to_fd_buf(rejectlogfd, g)) != g->ptr)
+    if (  (written_len = write_gstring_to_fd_buf(rejectlogfd, g))
+       != gstring_length(g))
       {
       log_write_failed(US"reject log", g->ptr, written_len);
       /* That function does not return */
@@ -1265,9 +1288,11 @@ if (flags & LOG_PANIC)
     panic_recurseflag = FALSE;
 
     if (panic_save_buffer)
-      (void) write(paniclogfd, panic_save_buffer, Ustrlen(panic_save_buffer));
+      if (write(paniclogfd, panic_save_buffer, Ustrlen(panic_save_buffer)) < 0)
+	DEBUG(D_any) debug_printf("sucks");
 
-    if ((written_len = write_gstring_to_fd_buf(paniclogfd, g)) != g->ptr)
+    if (  (written_len = write_gstring_to_fd_buf(paniclogfd, g))
+       != gstring_length(g))
       {
       int save_errno = errno;
       write_syslog(LOG_CRIT, log_buffer);
@@ -1290,6 +1315,30 @@ if (flags & LOG_PANIC)
   }
 }
 
+/* The public interface */
+
+void
+log_write(unsigned int selector, int flags, const char * format, ...)
+{
+va_list ap;
+va_start(ap, format);
+log_vwrite(selector, flags, format, ap);
+va_end(ap);
+}
+
+/* As the above, but adds in LOG_PANIC_DIE.
+We have this as a wripper so that we can mark it as never returning,
+for the benefit of static analysers. */
+
+void
+log_write_die(unsigned int selector, int flags, const char * format, ...)
+{
+va_list ap;
+va_start(ap, format);
+log_vwrite(selector, flags | LOG_PANIC_DIE, format, ap);
+UNREACHABLE;
+}
+
 
 
 /*************************************************
@@ -1404,7 +1453,7 @@ else for(;;)
   if (*string != '+' && *string != '-')
     {
     errmsg = string_sprintf("malformed %s_selector setting: "
-      "+ or - expected but found \"%s\"", which, string);
+      "+ or - expected but found %q", which, string);
     goto ERROR_RETURN;
     }
 
@@ -1466,7 +1515,7 @@ if (Ustrcmp(which, "debug") == 0)
   fprintf(stderr, "exim: %s\n", errmsg);
   exim_exit(EXIT_FAILURE);
   }
-else log_write(0, LOG_CONFIG|LOG_PANIC_DIE, "%s", errmsg);
+else log_write_die(0, LOG_CONFIG, "%s", errmsg);
 }
 
 
@@ -1497,7 +1546,7 @@ debug_logging_activate(const uschar * tag_name, const uschar * opts)
 if (debug_file)
   {
   debug_printf("DEBUGGING ACTIVATED FROM WITHIN CONFIG.\n"
-      "DEBUG: Tag=\"%s\" opts=\"%s\"\n", tag_name, opts ? opts : US"");
+      "DEBUG: Tag=%q opts=%q\n", tag_name, opts ? opts : US"");
   return;
   }
 
@@ -1533,7 +1582,7 @@ debug_logging_from_spool(const uschar * filename)
 {
 if (debug_fd < 0)
   {
-  Ustrncpy(debuglog_name, filename, sizeof(debuglog_name));
+  Ustrncpy(debuglog_name, filename, sizeof(debuglog_name)-1);
   if ((debug_fd = log_open_as_exim(filename)) >= 0)
     debug_file = fdopen(debug_fd, "w");
   DEBUG(D_deliver) debug_printf("debug enabled by spoolfile\n");
