@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -101,7 +101,10 @@ and eldapm_find(), with a difference in the "search_type" argument.
 
 The case of eldapauth_find() is special in that all it does is do
 authentication, returning OK or FAIL as appropriate. This isn't used as a
-lookup. Instead, it is called from expand.c as an expansion condition test.
+lookup. Instead, it is called via the generic search interface from expand.c
+as an expansion condition test.  We take a non/NULL return string as OK/FAIL.
+We do not advertise or document it as a general search method,
+but probably could.
 
 The DN from a successful lookup is placed in $ldap_dn. This feature postdates
 the provision of the SEARCH_LDAP_DN facility for returning just the DN as the
@@ -114,7 +117,7 @@ Arguments:
   search_type   SEARCH_LDAP_MULTIPLE allows values from multiple entries
                 SEARCH_LDAP_SINGLE allows values from one entry only
                 SEARCH_LDAP_DN gets the DN from one entry
-  res           set to point at the result (not used for ldapauth)
+  res           set to point at the result (ldapauth gets an empty string if OK)
   errmsg        set to point a message if result is not OK
   defer_break   set TRUE if no more servers to be tried after a DEFER
   user          user name for authentication, or NULL
@@ -166,7 +169,7 @@ BOOL   attribute_found = FALSE;
 BOOL   ldapi = FALSE;
 
 DEBUG(D_lookup) debug_printf_indent("perform_ldap_search:"
-    " ldap%s URL = \"%s\" server=%s port=%d "
+    " ldap%s URL = %q server=%s port=%d "
     "sizelimit=%d timelimit=%d tcplimit=%d\n",
     search_type == SEARCH_LDAP_MULTIPLE ? "m" :
     search_type == SEARCH_LDAP_DN       ? "dn" :
@@ -178,7 +181,7 @@ library that is in use doesn't recognize, say, "ldapi", it will barf here. */
 
 if (!ldap_is_ldap_url(CS ldap_url))
   {
-  *errmsg = string_sprintf("ldap_is_ldap_url: not an LDAP url \"%s\"\n",
+  *errmsg = string_sprintf("ldap_is_ldap_url: not an LDAP url %q\n",
     ldap_url);
   goto RETURN_ERROR_BREAK;
   }
@@ -187,7 +190,7 @@ if (!ldap_is_ldap_url(CS ldap_url))
 
 if ((rc = ldap_url_parse(CS ldap_url, &ludp)) != 0)
   {
-  *errmsg = string_sprintf("ldap_url_parse: (error %d) parsing \"%s\"\n", rc,
+  *errmsg = string_sprintf("ldap_url_parse: (error %d) parsing %q\n", rc,
     ldap_url);
   goto RETURN_ERROR_BREAK;
   }
@@ -236,7 +239,7 @@ if (host)
 #if defined LDAP_LIB_OPENLDAP2
   else if (strncmp(ludp->lud_scheme, "ldapi", 5) == 0)
     {
-    *errmsg = string_sprintf("ldapi requires an absolute path (\"%s\" given)",
+    *errmsg = string_sprintf("ldapi requires an absolute path (%q given)",
       host);
     goto RETURN_ERROR;
     }
@@ -322,7 +325,7 @@ if (!lcp)
 
   else
     {
-    uschar * init_ptr = Ustrchr(ldap_url, '/');
+    const uschar * init_ptr = Ustrchr(ldap_url, '/');
     g = string_catn(NULL, ldap_url, init_ptr - ldap_url);
     g = string_fmt_append(g, "//%s:%d/", shost, port);
     }
@@ -334,7 +337,7 @@ if (!lcp)
     DEBUG(D_lookup) debug_printf_indent("ldap_initialize with URL %s\n", s);
     if ((rc = ldap_initialize(&ld, CS s)) != LDAP_SUCCESS)
       {
-      *errmsg = string_sprintf("ldap_initialize: (error %d) URL \"%s\"\n",
+      *errmsg = string_sprintf("ldap_initialize: (error %d) URL %q\n",
 	rc, s);
       goto RETURN_ERROR;
       }
@@ -608,6 +611,7 @@ if (  !lcp->bound
 if (search_type == SEARCH_LDAP_AUTH)
   {
   DEBUG(D_lookup) debug_printf_indent("Bind succeeded: ldapauth returns OK\n");
+  *res = US"";
   goto RETURN_OK;
   }
 
@@ -1305,7 +1309,8 @@ return(control_ldap_search(ldap_url, SEARCH_LDAP_DN, result, errmsg));
 
 int
 eldapauth_find(void * handle, const uschar * filename, const uschar * ldap_url,
-  int length, uschar ** result, uschar ** errmsg, uint * do_cache)
+  int length, uschar ** result, uschar ** errmsg, uint * do_cache,
+  const uschar * opts)
 {
 return(control_ldap_search(ldap_url, SEARCH_LDAP_AUTH, result, errmsg));
 }
@@ -1403,7 +1408,7 @@ Arguments:
   s          the string to be quoted
   opt        additional option text or NULL if none
              only "dn" is recognized
-  idx	     lookup type index
+  idx	     quoter identification number
 
 Returns:     the processed string or NULL for a bad option
 */
@@ -1462,7 +1467,7 @@ while ((c = *t++))
 
 /* Get sufficient store to hold the quoted string */
 
-t = quoted = store_get_quoted(len + count + 1, s, idx);
+t = quoted = store_get_quoted(len + count + 1, s, idx, US"ldap");
 
 /* Handle plain quote_ldap */
 
@@ -1559,6 +1564,7 @@ gstring *
 ldap_version_report(gstring * g)
 {
 #ifdef DYNLOOKUP
+/*XXX it would be nice to haul a version string for the underlying ldap library */
 g = string_fmt_append(g, "Library version: LDAP: Exim version %s\n", EXIM_VERSION_STR);
 #endif
 return g;
@@ -1601,11 +1607,28 @@ static lookup_info ldapm_lookup_info = {
   .version_report = NULL                           /* no version reporting (redundant) */
 };
 
+static lookup_info ldapauth_lookup_info = {
+  .name = US"ldapauth",			/* lookup name */
+  .type = lookup_querystyle,		/* query-style lookup */
+  .open = eldap_open,			/* sic */    /* open function */
+  .check = NULL,			/* check function */
+  .find = eldapauth_find,		/* find function */
+  .close = NULL,			/* no close function */
+  .tidy = eldap_tidy,			/* sic */    /* tidy function */
+  .quote = NULL,			/* NO quoting function */
+  .version_report = NULL                           /* no version reporting (redundant) */
+};
+
 #ifdef DYNLOOKUP
 #define ldap_lookup_module_info _lookup_module_info
 #endif
 
-static lookup_info *_lookup_list[] = { &ldap_lookup_info, &ldapdn_lookup_info, &ldapm_lookup_info };
-lookup_module_info ldap_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 3 };
+static lookup_info *_lookup_list[] = {
+  &ldap_lookup_info,
+  &ldapdn_lookup_info,
+  &ldapm_lookup_info,
+  &ldapauth_lookup_info,
+  };
+lookup_module_info ldap_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 4 };
 
 /* End of lookups/ldap.c */
