@@ -2,11 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
 
+#ifndef MACROS_H
+#define MACROS_H
 
 /* These two macros make it possible to obtain the result of macro-expanding
 a string as a text string. This is sometimes useful for debugging output. */
@@ -47,6 +49,7 @@ allow tabs & spaces. A macro just makes life a bit easier. */
 #define string_printing(s) string_printing2((s), 0)
 #define SP_TAB		BIT(0)
 #define SP_SPACE	BIT(1)
+#define SP_DQUOTES	BIT(2)
 
 
 /* We need a special return code for "no recipients and failed to send an error
@@ -80,15 +83,6 @@ as unsigned. */
   ((uschar)(c) > 127 && print_topbitchars))
 
 
-/* When built with TLS support, the act of flushing SMTP output becomes
-a no-op once an SSL session is in progress. */
-
-#ifndef DISABLE_TLS
-#define mac_smtp_fflush() if (tls_in.active.sock < 0) fflush(smtp_out);
-#else
-#define mac_smtp_fflush() fflush(smtp_out);
-#endif
-
 
 /* Define which ends of pipes are for reading and writing, as some systems
 don't make the file descriptors two-way. */
@@ -102,7 +96,7 @@ don't make the file descriptors two-way. */
 
 /* A macro to simplify testing bits in lookup types */
 
-#define mac_islookup(a,b) ((lookup_list[a]->type & (b)) != 0)
+#define mac_islookup(li,b) ((li)->type & (b))
 
 /* Debugging control */
 
@@ -111,6 +105,13 @@ don't make the file descriptors two-way. */
 #define DEBUG(x)	if (IS_DEBUG(x))
 #define HDEBUG(x)	if (host_checking || IS_DEBUG(x))
 
+#define EARLY_DEBUG(x, fmt, ...) \
+  if (debug_startup) \
+    if (debug_fd < 0) \
+      fprintf(stderr, "%s", string_sprintf(fmt, __VA_ARGS__)); \
+    else DEBUG(x) \
+      debug_printf_indent(fmt, __VA_ARGS__);
+
 /* The default From: text for DSNs */
 
 #define DEFAULT_DSN_FROM "Mail Delivery System <Mailer-Daemon@$qualify_domain>"
@@ -343,7 +344,7 @@ and the first word of the log selector. */
 
 #define BIT_CLEAR(s,z,n) ((s)[BITWORD(n)] &= ~BITMASK(n))
 #define BIT_SET(s,z,n)   ((s)[BITWORD(n)] |=  BITMASK(n))
-#define BIT_TEST(s,z,n) (((s)[BITWORD(n)] &   BITMASK(n)) != 0)
+#define BIT_TEST(s,z,n)	 ((s)[BITWORD(n)]  &  BITMASK(n))
 
 /* Used in globals.c for initializing bit_table structures. T will be either
 D or L corresponding to the debug and log selector bits declared below. */
@@ -501,6 +502,7 @@ enum logbit {
   Li_subject,
   Li_tls_certificate_verified,
   Li_tls_cipher,
+  Li_tls_on_connect,
   Li_tls_peerdn,
   Li_tls_resumption,
   Li_tls_sni,
@@ -704,7 +706,7 @@ can be easily tested as a group. That is the only use of opt_bool_last. */
 enum { opt_bit = 32, opt_bool_verify, opt_bool_set, opt_expand_bool,
   opt_bool_last,
   opt_rewrite, opt_timelist, opt_uid, opt_gid, opt_uidlist, opt_gidlist,
-  opt_expand_uid, opt_expand_gid, opt_func, opt_void };
+  opt_expand_uid, opt_expand_gid, opt_func, opt_void, opt_module };
 
 /* There's a high-ish bit which is used to flag duplicate options, kept
 for compatibility, which shouldn't be output. Also used for hidden options
@@ -739,10 +741,12 @@ enum { v_none, v_sender, v_recipient, v_expn };
 #define vopt_callout_random       0x0020   /* during callout */
 #define vopt_callout_no_cache     0x0040   /* disable callout cache */
 #define vopt_callout_recipsender  0x0080   /* use real sender to verify recip */
-#define vopt_callout_recippmaster 0x0100   /* use postmaster to verify recip */
-#define vopt_callout_hold	  0x0200   /* lazy close connection */
-#define vopt_success_on_redirect  0x0400
-#define vopt_quota                0x0800   /* quota check, to local/appendfile */
+#define vopt_callout_r_pmaster	  0x0100   /* use postmaster to verify recip */
+#define vopt_callout_r_tptsender  0x0200   /* use s from tpt to verify recip */
+#define vopt_callout_hold	  0x0400   /* lazy close connection */
+#define vopt_success_on_redirect  0x0800
+#define vopt_quota                0x1000   /* quota check, to local/appendfile */
+#define vopt_atrn		  0x2000   /* ATRN customer mode */
 
 /* Values for fields in callout cache records */
 
@@ -822,10 +826,11 @@ local_scan.h */
 #define DEBUG_FROM_CONFIG       0x0001
 
 /* SMTP command identifiers for the smtp_connection_had field that records the
-most recent SMTP commands. SCH_NONE is "empty". */
+most recent SMTP commands. SCH_NONE is "empty".  The smtp_names array must have
+an entry for each. */
 
 enum { SCH_NONE, SCH_AUTH, SCH_DATA, SCH_BDAT,
-       SCH_EHLO, SCH_ETRN, SCH_EXPN, SCH_HELO,
+       SCH_EHLO, SCH_ATRN, SCH_ETRN, SCH_EXPN, SCH_HELO,
        SCH_HELP, SCH_MAIL, SCH_NOOP, SCH_QUIT, SCH_RCPT, SCH_RSET, SCH_STARTTLS,
        SCH_VRFY,
 #ifndef DISABLE_WELLKNOWN
@@ -954,20 +959,21 @@ order without checking carefully!
 **** IMPORTANT ****
 */
 
-enum { ACL_WHERE_RCPT,       /* Some controls are for RCPT only */
-       ACL_WHERE_MAIL,       /* )                                           */
-       ACL_WHERE_PREDATA,    /* ) There are several tests for "in message", */
-       ACL_WHERE_MIME,       /* ) implemented by <= WHERE_NOTSMTP           */
-       ACL_WHERE_DKIM,       /* )                                           */
-       ACL_WHERE_DATA,       /* )                                           */
+enum { ACL_WHERE_RCPT,		/* Some controls are for RCPT only */
+       ACL_WHERE_MAIL,		/* )						*/
+       ACL_WHERE_PREDATA,	/* ) There are several tests for "in message",	*/
+       ACL_WHERE_MIME,		/* ) implemented by <= WHERE_NOTSMTP		*/
+       ACL_WHERE_DKIM,		/* )						*/
+       ACL_WHERE_DATA,		/* )						*/
 #ifndef DISABLE_PRDR
-       ACL_WHERE_PRDR,       /* )                                           */
+       ACL_WHERE_PRDR,		/* )						*/
 #endif
-       ACL_WHERE_NOTSMTP,    /* )                                           */
+       ACL_WHERE_NOTSMTP,	/* )						*/
 
-       ACL_WHERE_AUTH,       /* These remaining ones are not currently    */
-       ACL_WHERE_CONNECT,    /* required to be in a special order so they */
-       ACL_WHERE_ETRN,       /* are just alphabetical.                    */
+       ACL_WHERE_AUTH,		/* These remaining ones are not currently	*/
+       ACL_WHERE_ATRN,		/* */
+       ACL_WHERE_CONNECT,	/* required to be in a special order so they	*/
+       ACL_WHERE_ETRN,		/* are just alphabetical.			*/
        ACL_WHERE_EXPN,
        ACL_WHERE_HELO,
        ACL_WHERE_MAILAUTH,
@@ -998,6 +1004,7 @@ enum { ACL_WHERE_RCPT,       /* Some controls are for RCPT only */
 #define ACL_BIT_NOTSMTP		BIT(ACL_WHERE_NOTSMTP)
 #define ACL_BIT_AUTH		BIT(ACL_WHERE_AUTH)
 #define ACL_BIT_CONNECT		BIT(ACL_WHERE_CONNECT)
+#define ACL_BIT_ATRN		BIT(ACL_WHERE_ATRN)
 #define ACL_BIT_ETRN		BIT(ACL_WHERE_ETRN)
 #define ACL_BIT_EXPN		BIT(ACL_WHERE_EXPN)
 #define ACL_BIT_HELO		BIT(ACL_WHERE_HELO)
@@ -1189,6 +1196,11 @@ typedef unsigned mcs_flags;
 #define SR_FINAL	TRUE
 #define SR_NOT_FINAL	FALSE
 
+/* Flags for continued-TLS-connection */
+#define CTF_CV	BIT(0)
+#define CTF_DV	BIT(1)
+#define CTF_TR	BIT(2)
+
 /* Return codes for smtp_write_mail_and_rcpt_cmds() */
 typedef enum {
   sw_mrc_ok,	/* good, rcpt results in addr->transport_return (PENDING_OK, DEFER, FAIL) */
@@ -1207,8 +1219,27 @@ When doing en extended loop of matching, release store periodically. */
 /* Debug an option access. Use for non-list ones about to be expanded
 (lists have their own debugging, under D_list). */
 #define GET_OPTION(name) \
-  DEBUG(D_expand) debug_printf("try option " name "\n");
+  DEBUG(D_expand) debug_printf_indent("try option '" name "'\n");
 
+
+#ifdef EXPERIMENTAL_SRV_SMTPS
+/* Service name for the feature; used in DNS SRV RRs */
+# define SRV_SMTPS_SERVICE_NAME	"smtp-tls"	/* Per version 5 draft */
+#endif
+
+/* Arg for smtp_fflush */
+#define SFF_UNCORK	TRUE
+#define SFF_NO_UNCORK	FALSE
+
+/* Arg for wouldblock_reading */
+#define WBR_DATA_OR_EOF	TRUE
+#define WBR_DATA_ONLY	FALSE
+
+/* Secondary-result bits for macros_expand() */
+#define MACRO_FOUND	BIT(0)	/* At least one macro was found in the line */
+#define MACRO_FIRST	BIT(1)	/* A macro was the first thing in the line */
+
+#endif	/* whole file */
 /* End of macros.h */
 /* vi: aw ai sw=2
 */
