@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2023 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -69,30 +69,31 @@ Returns:   pointer past the end of the address
 */
 
 uschar *
-parse_find_address_end(const uschar *s, BOOL nl_ends)
+parse_find_address_end(const uschar * s, BOOL nl_ends)
 {
 BOOL source_routing = *s == '@';
-int no_term = source_routing? 1 : 0;
+int no_term = source_routing ? 1 : 0;
 
-while (*s != 0 && (*s != ',' || no_term > 0) && (*s != '\n' || !nl_ends))
+while (*s && (*s != ',' || no_term > 0) && (*s != '\n' || !nl_ends))
   {
   /* Skip single quoted characters. Strictly these should not occur outside
   quoted strings in RFC 822 addresses, but they can in RFC 821 addresses. Pity
   about the lack of consistency, isn't it? */
 
-  if (*s == '\\' && s[1] != 0) s += 2;
+  if (*s == '\\' && s[1])
+    s += 2;
 
   /* Skip quoted items that are not inside brackets. Note that
   quoted pairs are allowed inside quoted strings. */
 
   else if (*s == '\"')
-    {
-    while (*(++s) != 0 && (*s != '\n' || !nl_ends))
+    while (*++s && (*s != '\n' || !nl_ends))
       {
-      if (*s == '\\' && s[1] != 0) s++;
-        else if (*s == '\"') { s++; break; }
+      if (*s == '\\' && s[1])
+	s++;
+      else if (*s == '\"')
+	{ s++; break; }
       }
-    }
 
   /* Skip comments, which may include nested brackets, but quotes
   are not recognized inside comments, though quoted pairs are. */
@@ -100,12 +101,13 @@ while (*s != 0 && (*s != ',' || no_term > 0) && (*s != '\n' || !nl_ends))
   else if (*s == '(')
     {
     int level = 1;
-    while (*(++s) != 0 && (*s != '\n' || !nl_ends))
-      {
-      if (*s == '\\' && s[1] != 0) s++;
-        else if (*s == '(') level++;
-          else if (*s == ')' && --level <= 0) { s++; break; }
-      }
+    while (*++s && (*s != '\n' || !nl_ends))
+      if (*s == '\\' && s[1])
+	s++;
+      else if (*s == '(')
+	level++;
+      else if (*s == ')' && --level <= 0)
+	{ s++; break; }
     }
 
   /* Non-special character; just advance. Passing the colon in a source
@@ -117,10 +119,12 @@ while (*s != 0 && (*s != ',' || no_term > 0) && (*s != '\n' || !nl_ends))
     if (*s == '<')
       {
       source_routing = s[1] == '@';
-      no_term = source_routing? 2 : 1;
+      no_term = source_routing ? 2 : 1;
       }
-    else if (*s == '>') no_term--;
-    else if (source_routing && *s == ':') no_term--;
+    else if (*s == '>')
+      no_term--;
+    else if (source_routing && *s == ':')
+      no_term--;
     s++;
     }
   }
@@ -565,7 +569,7 @@ s = read_local_part(s, t, errorptr, FALSE);
 if (*errorptr == NULL)
   if (*s != term)
     if (*s != '@')
-      *errorptr = string_sprintf("\"@\" or \".\" expected after \"%s\"", t);
+      *errorptr = string_sprintf("\"@\" or \".\" expected after %q", t);
     else
       {
       t += Ustrlen((const uschar *)t);
@@ -628,8 +632,8 @@ Returns:      points to the extracted address, or NULL on error
 #define FAILED(s) { *errorptr = s; goto PARSE_FAILED; }
 
 uschar *
-parse_extract_address(const uschar *mailbox, uschar **errorptr, int *start, int *end,
-  int *domain, BOOL allow_null)
+parse_extract_address(const uschar * mailbox, uschar ** errorptr,
+  int * start, int * end, int * domain, BOOL allow_null)
 {
 uschar * yield = store_get(Ustrlen(mailbox) + 1, mailbox);
 const uschar *startptr, *endptr;
@@ -878,7 +882,6 @@ const uschar *
 parse_quote_2047(const uschar * string, int len, const uschar * charset,
   BOOL fold)
 {
-const uschar * s = string;
 int hlen, line_off;
 BOOL coded = FALSE;
 BOOL first_byte = FALSE;
@@ -887,7 +890,7 @@ gstring * g =
 
 line_off = hlen;
 
-for (s = string; len > 0; s++, len--)
+for (const uschar * s = string; len > 0; s++, len--)
   {
   int ch = *s;
 
@@ -978,11 +981,10 @@ Returns:       the fixed RFC822 phrase
 const uschar *
 parse_fix_phrase(const uschar *phrase, int len)
 {
-int ch, i;
+int i;
 BOOL quoted = FALSE;
-const uschar *s, *end;
-uschar * buffer;
-uschar *t, *yield;
+const uschar * s, * end;
+uschar * buffer, * t, * yield;
 
 while (len > 0 && isspace(*phrase)) { phrase++; len--; }
 
@@ -1008,7 +1010,7 @@ yield = t = buffer + 1;
 
 while (s < end)
   {
-  ch = *s++;
+  int ch = *s++;
 
   /* Copy over quoted strings, remembering we encountered one */
 
@@ -1353,16 +1355,16 @@ for (;;)
 
   if (special)
     {
-    uschar * ss = Ustrchr(s+1, ':') + 1; /* line after the special... */
+    uschar * p = Ustrchr(s+1, ':') + 1; /* line after the special... */
     if ((options & specopt) == specbit)
       {
       *error = string_sprintf("\"%.*s\" is not permitted", len, s);
       return FF_ERROR;
       }
-    while (*ss && isspace(*ss)) ss++;	/* skip leading whitespace */
-    if ((len = Ustrlen(ss)) > 0)	/* ignore trailing newlines */
-      for (const uschar * t = ss + len - 1; t >= ss && *t == '\n'; t--) len--;
-    *error = string_copyn(ss, len);	/* becomes the error */
+    Uskip_whitespace(&p);		/* skip leading whitespace */
+    if ((len = Ustrlen(p)) > 0)		/* ignore trailing newlines */
+      for (const uschar * t = p + len - 1; t >= p && *t == '\n'; t--) len--;
+    *error = string_copyn(p, len);	/* becomes the error */
     return special;
     }
 
@@ -1373,13 +1375,10 @@ for (;;)
 
   if (Ustrncmp(s, ":include:", 9) == 0)
     {
-    uschar * filebuf;
-    uschar filename[256];
+    uschar * filebuf, * filename;
     const uschar * t = s+9;
-    int flen = len - 9;
-    int frc;
+    int flen = len - 9, frc;
     struct stat statbuf;
-    address_item * last;
     FILE * f;
 
     while (flen > 0 && isspace(*t)) { t++; flen--; }
@@ -1390,20 +1389,19 @@ for (;;)
       return FF_ERROR;
       }
 
-    if (flen > sizeof(filename)-1)
+    if (flen > 255)
       {
-      *error = string_sprintf("included file name \"%s\" is too long", t);
+      *error = string_sprintf("included file name %q is too long", t);
       return FF_ERROR;
       }
 
-    Ustrncpy(filename, t, flen);
-    filename[flen] = 0;
+    filename = string_copyn(t, flen);
 
     /* Insist on absolute path */
 
     if (filename[0] != '/')
       {
-      *error = string_sprintf("included file \"%s\" is not an absolute path",
+      *error = string_sprintf("included file %q is not an absolute path",
         filename);
       return FF_ERROR;
       }
@@ -1427,12 +1425,14 @@ for (;;)
 
     if (directory)
       {
-      int len = Ustrlen(directory);
+      int dlen = Ustrlen(directory);
       uschar * p;
 
-      while (len > 0 && directory[len-1] == '/') len--;		/* ignore trailing '/' */
-      p = filename + len;
-      if (Ustrncmp(filename, directory, len) != 0 || *p != '/')
+      /* ignore trailing '/' */
+      while (dlen > 0 && directory[dlen-1] == '/') dlen--;
+
+      p = filename + dlen;
+      if (Ustrncmp(filename, directory, dlen) != 0 || *p != '/')
         {
         *error = string_sprintf("included file %s is not in directory %s",
           filename, directory);
@@ -1565,6 +1565,7 @@ for (;;)
 
     if (addr)
       {
+      address_item * last;
       for (last = addr; last->next; last = last->next) count++;
       last->next = *anchor;
       *anchor = addr;
@@ -1677,7 +1678,7 @@ for (;;)
           }
         else
           {
-          *error = string_sprintf("%s in \"%s\"", *error, s_ltd);
+          *error = string_sprintf("%s in %q", *error, s_ltd);
           return FF_ERROR;
           }
         }
@@ -2122,7 +2123,7 @@ while (Ufgets(buffer, sizeof(buffer), stdin) != NULL)
     uschar extract[1024];
     Ustrncpy(extract, buffer+start, end-start);
     extract[end-start] = 0;
-    printf("%s %d %d %d \"%s\"\n", out, start, end, domain, extract);
+    printf("%s %d %d %d %q\n", out, start, end, domain, extract);
     }
   }
 
@@ -2143,7 +2144,7 @@ while (Ufgets(buffer, sizeof(buffer), stdin) != NULL)
     uschar extract[1024];
     Ustrncpy(extract, buffer+start, end-start);
     extract[end-start] = 0;
-    printf("%s %d %d %d \"%s\"\n", out, start, end, domain, extract);
+    printf("%s %d %d %d %q\n", out, start, end, domain, extract);
     }
   }
 allow_utf8_domains = FALSE;
@@ -2174,7 +2175,7 @@ while (Ufgets(buffer, sizeof(buffer), stdin) != NULL)
       uschar extract[1024];
       Ustrncpy(extract, buffer+start, end-start);
       extract[end-start] = 0;
-      printf("%s %d %d %d \"%s\"\n", out, start, end, domain, extract);
+      printf("%s %d %d %d %q\n", out, start, end, domain, extract);
       }
 
     s = ss + (terminator? 1:0);
