@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -45,7 +45,7 @@ int dnslookup_router_options_count =
 
 /* Dummy entries */
 dnslookup_router_options_block dnslookup_router_option_defaults = {0};
-void dnslookup_router_init(router_instance *rblock) {}
+void dnslookup_router_init(driver_instance *rblock) {}
 int dnslookup_router_entry(router_instance *rblock, address_item *addr,
   struct passwd *pw, int verify, address_item **addr_local,
   address_item **addr_remote, address_item **addr_new,
@@ -56,21 +56,12 @@ int dnslookup_router_entry(router_instance *rblock, address_item *addr,
 
 
 
-/* Default private options block for the dnslookup router. */
+/* Default private options block for the dnslookup router.
+All non-mentioned are NULL/0/FALSE. */
 
 dnslookup_router_options_block dnslookup_router_option_defaults = {
-  .check_secondary_mx =	FALSE,
   .qualify_single =	TRUE,
-  .search_parents =	FALSE,
   .rewrite_headers =	TRUE,
-  .widen_domains =	NULL,
-  .mx_domains =		NULL,
-  .mx_fail_domains =	NULL,
-  .srv_fail_domains =	NULL,
-  .check_srv =		NULL,
-  .fail_defer_domains =	NULL,
-  .ipv4_only =		NULL,
-  .ipv4_prefer =	NULL,
 };
 
 
@@ -83,7 +74,7 @@ dnslookup_router_options_block dnslookup_router_option_defaults = {
 consistency checks to be done, or anything else that needs to be set up. */
 
 void
-dnslookup_router_init(router_instance *rblock)
+dnslookup_router_init(driver_instance *rblock)
 {
 /*
 dnslookup_router_options_block *ob =
@@ -154,15 +145,16 @@ int rc;
 int widen_sep = 0;
 int whichrrs = HOST_FIND_BY_MX | HOST_FIND_BY_A | HOST_FIND_BY_AAAA;
 dnslookup_router_options_block * ob =
-  (dnslookup_router_options_block *)(rblock->options_block);
-uschar * srv_service = NULL, * widen = NULL;
+  (dnslookup_router_options_block *)(rblock->drinst.options_block);
+const uschar * srv_service = NULL;
+uschar * widen = NULL;
 const uschar * pre_widen = addr->domain, * post_widen = NULL;
 const uschar * fully_qualified_name, * listptr;
 uschar widen_buffer[256];
 
 DEBUG(D_route)
-  debug_printf("%s router called for %s\n  domain = %s\n",
-    rblock->name, addr->address, addr->domain);
+  debug_printf_indent("%s router called for %s\n  domain = %s\n",
+    rblock->drinst.name, addr->address, addr->domain);
 
 /* If an SRV check is required, expand the service name */
 
@@ -171,8 +163,8 @@ if (ob->check_srv)
   if (  !(srv_service = expand_string(ob->check_srv))
      && !f.expand_string_forcedfail)
     {
-    addr->message = string_sprintf("%s router: failed to expand \"%s\": %s",
-      rblock->name, ob->check_srv, expand_string_message);
+    addr->message = string_sprintf("%s router: failed to expand %q: %s",
+      rblock->drinst.name, ob->check_srv, expand_string_message);
     return DEFER;
     }
   else
@@ -237,27 +229,27 @@ for (;;)
     /* not expanded so should never be tainted */
     widen = string_nextinlist(&listptr, &widen_sep, widen_buffer,
       sizeof(widen_buffer));
-    DEBUG(D_route) debug_printf("%s router widened %s to %s\n", rblock->name,
-      addr->domain, h.name);
+    DEBUG(D_route) debug_printf("%s router widened %s to %s\n",
+      rblock->drinst.name, addr->domain, h.name);
     }
   else if (post_widen)
     {
     h.name = post_widen;
     post_widen = NULL;
     DEBUG(D_route) debug_printf("%s router trying %s after widening failed\n",
-      rblock->name, h.name);
+      rblock->drinst.name, h.name);
     }
   else return DECLINE;
 
   /* Check if we must request only. or prefer, ipv4 */
 
   if (  ob->ipv4_only
-     && expand_check_condition(ob->ipv4_only, rblock->name, US"router"))
+     && expand_check_condition(ob->ipv4_only, rblock->drinst.name, US"router"))
     flags = flags & ~HOST_FIND_BY_AAAA | HOST_FIND_IPV4_ONLY;
   else if (f.search_find_defer)
     return DEFER;
   if (  ob->ipv4_prefer
-     && expand_check_condition(ob->ipv4_prefer, rblock->name, US"router"))
+     && expand_check_condition(ob->ipv4_prefer, rblock->drinst.name, US"router"))
     flags |= HOST_FIND_IPV4_FIRST;
   else if (f.search_find_defer)
     return DEFER;
@@ -273,6 +265,7 @@ for (;;)
   h.mx = MX_NONE;
   h.status = hstatus_unknown;
   h.why = hwhy_unknown;
+  h.dnssec_used = DS_UNK;
   h.last_try = 0;
 
   /* Unfortunately, we cannot set the mx_only option in advance, because the
@@ -291,8 +284,7 @@ for (;;)
     expand_level++;
     rc = host_find_bydns(&h, CUS rblock->ignore_target_hosts, flags,
       srv_service, ob->srv_fail_domains, ob->mx_fail_domains,
-      &rblock->dnssec,
-      &fully_qualified_name, &removed);
+      &rblock->dnssec, &fully_qualified_name, &removed);
     expand_level--;
    }
 
@@ -314,7 +306,7 @@ for (;;)
 
       case OK:
       DEBUG(D_route) debug_printf("%s router rejected %s: no MX record(s)\n",
-        rblock->name, fully_qualified_name);
+        rblock->drinst.name, fully_qualified_name);
       continue;
       }
 
@@ -331,7 +323,7 @@ for (;;)
     if (rblock->pass_on_timeout)
       {
       DEBUG(D_route) debug_printf("%s router timed out, and pass_on_timeout is set\n",
-        rblock->name);
+        rblock->drinst.name);
       return PASS;
       }
     addr->message = US"host lookup did not complete";
@@ -351,7 +343,7 @@ for (;;)
 
       case OK:
 	DEBUG(D_route) debug_printf("%s router: matched fail_defer_domains\n",
-	  rblock->name);
+	  rblock->drinst.name);
 	addr->message = US"missing MX, or all MXs point to missing A records,"
 	  " and defer requested";
 	return DEFER;
@@ -394,7 +386,7 @@ for (;;)
 
   if (f.host_find_failed_syntax)
     {
-    addr->message = string_sprintf("mail domain \"%s\" is syntactically "
+    addr->message = string_sprintf("mail domain %q is syntactically "
       "invalid", h.name);
     return DECLINE;
     }
@@ -467,16 +459,40 @@ addr->host_list[0] = h;
 /* Fill in the transport and queue the address for delivery. */
 
 if (!rf_get_transport(rblock->transport_name, &(rblock->transport),
-      addr, rblock->name, NULL))
+      addr, rblock->drinst.name, NULL))
   return DEFER;
 
 addr->transport = rblock->transport;
 
-return rf_queue_add(addr, addr_local, addr_remote, rblock, pw)?
-  OK : DEFER;
+return rf_queue_add(addr, addr_local, addr_remote, rblock, pw) ?  OK : DEFER;
 }
 
+
+
+
+# ifdef DYNLOOKUP
+#  define dnslookup_router_info _router_info
+# endif
 #endif	/*!MACRO_PREDEF*/
+
+router_info dnslookup_router_info =
+{
+.drinfo = {
+  .driver_name =	US"dnslookup",
+  .options =		dnslookup_router_options,
+  .options_count =	&dnslookup_router_options_count,
+  .options_block =	&dnslookup_router_option_defaults,
+  .options_len =	sizeof(dnslookup_router_options_block),
+  .init =		dnslookup_router_init,
+# ifdef DYNLOOKUP
+  .dyn_magic =		ROUTER_MAGIC,
+# endif
+  },
+.code =			dnslookup_router_entry,
+.tidyup =		NULL,     /* no tidyup entry */
+.ri_flags =		ri_yestransport
+};
+
 #endif	/*ROUTER_DNSLOOKUP*/
 /* End of routers/dnslookup.c */
 /* vi: aw ai sw=2
