@@ -2,6 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -25,7 +26,7 @@ Returns:      nothing
 */
 
 void
-rf_set_ugid(address_item *addr, ugid_block *ugid)
+rf_set_ugid(address_item * addr, const ugid_block * ugid)
 {
 if (ugid->uid_set)
   {
