@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2015 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -45,54 +45,60 @@ static int open_filecount = 0;
 
 static rmark search_reset_point = NULL;
 
+/* Cache control bits for results cache */
+#define CACHE_RD	BIT(0)
+#define CACHE_WR	BIT(1)
 
 
 /*************************************************
 *      Validate a plain lookup type name         *
 *************************************************/
 
+static const lookup_info *
+internal_search_findtype(const uschar * name)
+{
+tree_node * t = tree_search(lookups_tree, name);
+return t ? t->data.ptr : NULL;
+}
+
 /* Only those names that are recognized and whose code is included in the
-binary give an OK response. Use a binary chop search now that the list has got
-so long.
+binary give an OK response. Types are held in a binary tree for fast location
+and dynamic insertion.  If not initially found, try to load a module if
+any were compiled.
 
 Arguments:
   name       lookup type name - not necessarily zero terminated (e.g. dbm*)
   len        length of the name
 
-Returns:     +ve => valid lookup name; value is offset in lookup_list
-             -ve => invalid name; message in search_error_message.
+Returns:     ptr to info struct for the lookup,
+	     or NULL with message in search_error_message.
 */
 
-int
+const lookup_info *
 search_findtype(const uschar * name, int len)
 {
-for (int bot = 0, top = lookup_list_count; top > bot; )
-  {
-  int mid = (top + bot)/2;
-  int c = Ustrncmp(name, lookup_list[mid]->name, len);
+const lookup_info * li;
 
-  /* If c == 0 we have matched the incoming name with the start of the search
-  type name. However, some search types are substrings of others (e.g. nis and
-  nisplus) so we need to check that the lengths are the same. The length of the
-  type name cannot be shorter (else c would not be 0); if it is not equal it
-  must be longer, and in that case, the incoming name comes before the name we
-  are testing. By leaving c == 0 when the lengths are different, and doing a
-  > 0 test below, this all falls out correctly. */
+if (name[len])
+  name = string_copyn(name, len);
+if ((li = internal_search_findtype(name)))
+  return li;
 
-  if (c == 0 && Ustrlen(lookup_list[mid]->name) == len)
-    {
-    if (lookup_list[mid]->find != NULL) return mid;
-    search_error_message  = string_sprintf("lookup type \"%.*s\" is not "
-      "available (not in the binary - check buildtime LOOKUP configuration)",
-      len, name);
-    return -1;
-    }
+#ifdef LOOKUP_MODULE_DIR
+    DEBUG(D_lookup)
+      debug_printf_indent("searchtype %s not initially found\n", name);
 
-  if (c > 0) bot = mid + 1; else top = mid;
-  }
+    if (lookup_one_mod_load(name, NULL))
+      if ((li = internal_search_findtype(name)))
+	return li;
+      else
+	{ DEBUG(D_lookup) debug_printf_indent("find retry failed\n"); }
+    else DEBUG(D_lookup)
+      debug_printf_indent("scan modules dir for %s failed\n", name);
+#endif
 
-search_error_message = string_sprintf("unknown lookup type \"%.*s\"", len, name);
-return -1;
+search_error_message  = string_sprintf("unknown lookup type %q", name);
+return NULL;
 }
 
 
@@ -117,18 +123,17 @@ Arguments:
   starflags    where to put the SEARCH_STAR and SEARCH_STARAT flags
   opts	       where to put the options
 
-Returns:     +ve => valid lookup name; value is offset in lookup_list
-             -ve => invalid name; message in search_error_message.
+Returns:     ptr to info struct for the lookup,
+	     or NULL with message in search_error_message.
 */
 
-int
+const lookup_info *
 search_findtype_partial(const uschar *name, int *ptypeptr, const uschar **ptypeaff,
   int *afflen, int *starflags, const uschar ** opts)
 {
-int len, stype;
-int pv = -1;
-const uschar *ss = name;
-const uschar * t;
+const lookup_info * li;
+int pv = -1, len;
+const uschar * ss = name, * t;
 
 *starflags = 0;
 *ptypeaff = NULL;
@@ -163,9 +168,9 @@ if (Ustrncmp(name, "partial", 7) == 0)
   else
     {
     BAD_TYPE:
-    search_error_message = string_sprintf("format error in lookup type \"%s\"",
+    search_error_message = string_sprintf("format error in lookup type %q",
       name);
-    return -1;
+    return NULL;
     }
   }
 
@@ -194,31 +199,31 @@ else
 binary are valid. For query-style types, "partial" and default types are
 erroneous. */
 
-stype = search_findtype(ss, len);
-if (stype >= 0 && mac_islookup(stype, lookup_querystyle))
+li = search_findtype(ss, len);
+if (li && mac_islookup(li, lookup_querystyle))
   {
   if (pv >= 0)
     {
     search_error_message = string_sprintf("\"partial\" is not permitted "
-      "for lookup type \"%s\"", ss);
-    return -1;
+      "for lookup type %q", ss);
+    return NULL;
     }
   if ((*starflags & (SEARCH_STAR|SEARCH_STARAT)) != 0)
     {
     search_error_message = string_sprintf("defaults using \"*\" or \"*@\" are "
-      "not permitted for lookup type \"%s\"", ss);
-    return -1;
+      "not permitted for lookup type %q", ss);
+    return NULL;
     }
   }
 
 *ptypeptr = pv;
-return stype;
+return li;
 }
 
 
 /* Set the parameters for the three different kinds of lookup.
 Arguments:
- search_type	the search-type code
+ li		the info block for the search type
  search		the search-type string
  query		argument for the search; filename or query
  fnamep		pointer to return filename
@@ -227,11 +232,11 @@ Arguments:
 Return:	keyquery	the search-type (for single-key) or query (for query-type)
  */
 uschar *
-search_args(int search_type, uschar * search, uschar * query, uschar ** fnamep,
-  const uschar * opts)
+search_args(const lookup_info * li, uschar * search, uschar * query,
+  uschar ** fnamep, const uschar * opts)
 {
 Uskip_whitespace(&query);
-if (mac_islookup(search_type, lookup_absfilequery))
+if (mac_islookup(li, lookup_absfilequery))
   {					/* query-style but with file (sqlite) */
   int sep = ',';
 
@@ -255,7 +260,7 @@ if (mac_islookup(search_type, lookup_absfilequery))
     *fnamep = NULL;
   return query;		/* remainder after file skipped */
   }
-if (!mac_islookup(search_type, lookup_querystyle))
+if (!mac_islookup(li, lookup_querystyle))
   {					/* single-key */
   *fnamep = query;
   return search;	/* modifiers important so use "keyquery" for them */
@@ -288,11 +293,19 @@ Returns:     nothing
 static void
 tidyup_subtree(tree_node *t)
 {
-search_cache * c = (search_cache *)(t->data.ptr);
+search_cache * c = (search_cache *)t->data.ptr;
 if (t->left)  tidyup_subtree(t->left);
 if (t->right) tidyup_subtree(t->right);
-if (c && c->handle && lookup_list[c->search_type]->close)
-  lookup_list[c->search_type]->close(c->handle);
+if (c && c->handle && c->li->close)
+  c->li->close(c->handle);
+}
+
+
+static void
+tidy_cb(uschar * name, uschar * ptr, void * ctx)
+{
+lookup_info * li = (lookup_info *)ptr;
+if (li->tidy) (li->tidy)();
 }
 
 
@@ -308,6 +321,7 @@ search_tidyup(void)
 int old_pool = store_pool;
 
 DEBUG(D_lookup) debug_printf_indent("search_tidyup called\n");
+expand_level++;
 
 /* Close individually each cached open file. */
 
@@ -322,11 +336,11 @@ open_filecount = 0;
 
 /* Call the general tidyup entry for any drivers that have one. */
 
-for (int i = 0; i < lookup_list_count; i++) if (lookup_list[i]->tidy)
-  (lookup_list[i]->tidy)();
+tree_walk(lookups_tree, tidy_cb, NULL);
 
 if (search_reset_point) search_reset_point = store_reset(search_reset_point);
 store_pool = old_pool;
+expand_level--;
 }
 
 
@@ -366,7 +380,7 @@ but is marked closed.
 Arguments:
   filename       the name of the file for single-key+file style lookups,
                  NULL for query-style lookups
-  search_type    the type of search required
+  li		 the info block for the type of search required
   modemask       if a real single file is used, this specifies mode bits that
                  must not be set; otherwise it is ignored
   owners         if a real single file is used, this specifies the possible
@@ -381,13 +395,12 @@ Returns:         an identifying handle for the open database;
 */
 
 void *
-search_open(const uschar * filename, int search_type, int modemask,
+search_open(const uschar * filename, const lookup_info * li, int modemask,
   uid_t * owners, gid_t * owngroups)
 {
-void *handle;
-tree_node *t;
-search_cache *c;
-lookup_info *lk = lookup_list[search_type];
+void * handle;
+tree_node * t;
+search_cache * c;
 uschar keybuffer[256];
 int old_pool = store_pool;
 
@@ -403,7 +416,7 @@ if (filename && is_tainted(filename))
 store_pool = POOL_SEARCH;
 if (!search_reset_point) search_reset_point = store_mark();
 
-DEBUG(D_lookup) debug_printf_indent("search_open: %s \"%s\"\n", lk->name,
+DEBUG(D_lookup) debug_printf_indent("search_open: %s %q\n", li->name,
   filename ? filename : US"NULL");
 
 /* See if we already have this open for this type of search, and if so,
@@ -411,14 +424,15 @@ pass back the tree block as the handle. The key for the tree node is the search
 type plus '0' concatenated with the file name. There may be entries in the tree
 with closed files if a lot of files have been opened. */
 
-sprintf(CS keybuffer, "%c%.254s", search_type + '0',
+sprintf(CS keybuffer, "%c%.254s", li->acq_num+ '0',
   filename ? filename : US"");
 
 if ((t = tree_search(search_tree, keybuffer)))
   {
   if ((c = (search_cache *)t->data.ptr)->handle)
     {
-    DEBUG(D_lookup) debug_printf_indent("  cached open\n");
+    DEBUG(D_lookup)
+      if (c->handle != (void *)1) debug_printf_indent("  cached open\n");
     store_pool = old_pool;
     return t;
     }
@@ -431,20 +445,20 @@ this, if the search type is one that uses real files, check on the number that
 we are holding open in the cache. If the limit is reached, close the least
 recently used one. */
 
-if (lk->type == lookup_absfile && open_filecount >= lookup_open_max)
+if (li->type == lookup_absfile && open_filecount >= lookup_open_max)
   if (!open_bot)
     log_write(0, LOG_MAIN|LOG_PANIC, "too many lookups open, but can't find "
       "one to close");
   else
     {
-    search_cache *c = (search_cache *)(open_bot->data.ptr);
+    search_cache * c = (search_cache *)(open_bot->data.ptr);
     DEBUG(D_lookup) debug_printf_indent("Too many lookup files open\n  closing %s\n",
       open_bot->name);
     if ((open_bot = c->up))
       ((search_cache *)(open_bot->data.ptr))->down = NULL;
     else
       open_top = NULL;
-    ((lookup_list[c->search_type])->close)(c->handle);
+    (c->li->close)(c->handle);
     c->handle = NULL;
     open_filecount--;
     }
@@ -452,24 +466,24 @@ if (lk->type == lookup_absfile && open_filecount >= lookup_open_max)
 /* If opening is successful, call the file-checking function if there is one,
 and if all is still well, enter the open database into the tree. */
 
-if (!(handle = (lk->open)(filename, &search_error_message)))
+if (!(handle = (li->open)(filename, &search_error_message)))
   {
   store_pool = old_pool;
   return NULL;
   }
 
-if (  lk->check
-   && !lk->check(handle, filename, modemask, owners, owngroups,
+if (  li->check
+   && !li->check(handle, filename, modemask, owners, owngroups,
 	 &search_error_message))
   {
-  lk->close(handle);
+  li->close(handle);
   store_pool = old_pool;
   return NULL;
   }
 
 /* If this is a search type that uses real files, keep count. */
 
-if (lk->type == lookup_absfile) open_filecount++;
+if (li->type == lookup_absfile) open_filecount++;
 
 /* If we found a previously opened entry in the tree, re-use it; otherwise
 insert a new entry. On re-use, leave any cached lookup data and the lookup
@@ -486,7 +500,7 @@ if (!t)
 else c = t->data.ptr;
 
 c->handle = handle;
-c->search_type = search_type;
+c->li = li;
 c->up = c->down = NULL;
 
 store_pool = old_pool;
@@ -505,13 +519,13 @@ return t;
 is cached.
 
 Arguments:
-  handle       the handle from search_open; points to tree node
-  filename     the filename that was handed to search_open, or
-               NULL for query-style searches
-  keystring    the keystring for single-key+file lookups, or
-               the querystring for query-style lookups
-  cache_rd     FALSE to avoid lookup in cache layer
-  opts	       type-specific options
+  handle	the handle from search_open; points to tree node
+  filename	the filename that was handed to search_open, or
+		NULL for query-style searches
+  keystring	the keystring for single-key+file lookups, or
+		the querystring for query-style lookups
+  cache		cache control bits (RD, WR)
+  opts		type-specific options
 
 Returns:       a pointer to a dynamic string containing the answer,
                or NULL if the query failed or was deferred; in the
@@ -520,14 +534,14 @@ Returns:       a pointer to a dynamic string containing the answer,
 */
 
 static uschar *
-internal_search_find(void * handle, const uschar * filename, uschar * keystring,
-  BOOL cache_rd, const uschar * opts)
+internal_search_find(void * handle, const uschar * filename,
+  const uschar * keystring, unsigned cache, const uschar * opts)
 {
 tree_node * t = (tree_node *)handle;
 search_cache * c = (search_cache *)(t->data.ptr);
+const lookup_info * li = c->li;
 expiring_data * e = NULL;	/* compiler quietening */
 uschar * data = NULL;
-int search_type = t->name[0] - '0';
 int old_pool = store_pool;
 
 /* Lookups that return DEFER may not always set an error message. So that
@@ -536,14 +550,13 @@ the callers don't have to test for NULL, set an empty string. */
 search_error_message = US"";
 f.search_find_defer = FALSE;
 
-DEBUG(D_lookup) debug_printf_indent("internal_search_find: file=\"%s\"\n  "
-  "type=%s key=\"%s\" opts=%s%s%s\n", filename,
-  lookup_list[search_type]->name, keystring,
-  opts ? "\"" : "", opts, opts ? "\"" : "");
+DEBUG(D_lookup) debug_printf_indent("internal_search_find: file=%q\n  "
+  "type=%s key=%q opts=%s%s%s\n", filename,
+  li->name, keystring, opts ? "\"" : "", opts, opts ? "\"" : "");
 
 /* Insurance. If the keystring is empty, just fail. */
 
-if (keystring[0] == 0) return NULL;
+if (!*keystring) return NULL;
 
 /* Use the special store pool for search data */
 
@@ -556,7 +569,7 @@ whether we want to use the cache entry last so that we can always replace it. */
 if (  (t = tree_search(c->item_cache, keystring))
    && (!(e = t->data.ptr)->expiry || e->expiry > time(NULL))
    && (!opts && !e->opts  ||  opts && e->opts && Ustrcmp(opts, e->opts) == 0)
-   && cache_rd
+   && cache & CACHE_RD
    )
   { /* Data was in the cache already; set the pointer from the tree node */
   data = e->data.ptr;
@@ -566,7 +579,7 @@ if (  (t = tree_search(c->item_cache, keystring))
   }
 else
   {
-  uint do_cache = UINT_MAX;
+  uint do_cache = cache & CACHE_WR ? UINT_MAX : 0;
   int keylength = Ustrlen(keystring);
 
   DEBUG(D_lookup)
@@ -574,11 +587,12 @@ else
     if (t)
       debug_printf_indent("cached data found but %s; ",
 	e->expiry && e->expiry <= time(NULL) ? "out-of-date"
-	: cache_rd ? "wrong opts" : "no_rd option set");
+	: cache & CACHE_RD ? "wrong opts" : "no_rd option set");
     debug_printf_indent("%s lookup required for %s%s%s\n",
       filename ? US"file" : US"database",
       keystring,
-      filename ? US"\n  in " : US"", filename ? filename : US"");
+      filename ? US"\n  in " : US"",
+      filename ? filename : US"");
     if (!filename && is_tainted(keystring))
       {
       debug_printf_indent("                             ");
@@ -591,11 +605,11 @@ else
 
   XXX Should we this move into lf_sqlperform() ?  The server-taint check is there.
   Also it already knows about looking for a "servers" spec in the query string.
-  Passing search_type down that far is an issue.
+  Passing required_quoter_id down that far is an issue.
   */
 
-  if (  !filename && lookup_list[search_type]->quote
-     && is_tainted(keystring) && !is_quoted_like(keystring, search_type))
+  if (  !filename && li->quote
+     && is_tainted(keystring) && !is_quoted_like(keystring, li))
     {
     const uschar * ks = keystring;
     uschar * loc = acl_current_verb();
@@ -627,10 +641,12 @@ else
 
     DEBUG(D_lookup)
       {
-      int q = quoter_for_address(ks);
-      debug_printf_indent("search_type %d (%s) quoting %d (%s)\n",
-	search_type, lookup_list[search_type]->name,
-	q, is_real_quoter(q) ? lookup_list[q]->name : US"none");
+      const uschar * quoter_name;
+      int q = quoter_for_address(ks, &quoter_name);
+
+      debug_printf_indent("required_quoter_id (%s) quoting %d (%s)\n",
+	li->name,
+	q, quoter_name);
       }
 #endif
     }
@@ -639,7 +655,7 @@ else
   like FAIL, except that search_find_defer is set so the caller can
   distinguish if necessary. */
 
-  if (lookup_list[search_type]->find(c->handle, filename, keystring, keylength,
+  if (li->find(c->handle, filename, keystring, keylength,
 	  &data, &search_error_message, &do_cache, opts) == DEFER)
     f.search_find_defer = TRUE;
 
@@ -672,14 +688,17 @@ else
     e->data.ptr = data;
     }
 
-/* If caching was disabled, empty the cache tree. We just set the cache
-pointer to NULL here, because we cannot release the store at this stage. */
+/* If caching was disabled by the method call (as opposed to a no_wr option),
+empty the cache tree. We just set the cache pointer to NULL here, because we
+cannot release the store at this stage. */
 
-  else
+  else if (cache & CACHE_WR)
     {
     DEBUG(D_lookup) debug_printf_indent("lookup forced cache cleanup\n");
     c->item_cache = NULL; 	/* forget all lookups on this connection */
     }
+  else DEBUG(D_lookup)
+    debug_printf_indent("no_wr option: no cache invalidate\n");
   }
 
 out:
@@ -729,18 +748,19 @@ Returns:         a pointer to a dynamic string containing the answer,
 */
 
 uschar *
-search_find(void * handle, const uschar * filename, uschar * keystring,
+search_find(void * handle, const uschar * filename, const uschar * keystring,
   int partial, const uschar * affix, int affixlen, int starflags,
   int * expand_setup, const uschar * opts)
 {
 tree_node * t = (tree_node *)handle;
-BOOL set_null_wild = FALSE, cache_rd = TRUE, ret_key = FALSE;
+BOOL set_null_wild = FALSE, ret_key = FALSE;
+unsigned cache = CACHE_RD | CACHE_WR;
 uschar * yield;
 
 DEBUG(D_lookup)
   {
   if (partial < 0) affixlen = 99;   /* So that "NULL" prints */
-  debug_printf_indent("search_find: file=\"%s\"\n  key=\"%s\" "
+  debug_printf_indent("search_find: file=%q\n  key=%q "
     "partial=%d affix=%.*s starflags=%x opts=%s%s%s\n",
     filename ? filename : US"NULL",
     keystring, partial, affixlen, affix, starflags,
@@ -757,9 +777,11 @@ if (opts)
   int sep = ',';
   gstring * g = NULL;
 
-  for (uschar * ele; ele = string_nextinlist(&opts, &sep, NULL, 0); )
-    if (Ustrcmp(ele, "ret=key") == 0) ret_key = TRUE;
-    else if (Ustrcmp(ele, "cache=no_rd") == 0) cache_rd = FALSE;
+  for (const uschar * ele; ele = string_nextinlist(&opts, &sep, NULL, 0); )
+    if (Ustrcmp(ele, "ret=key") == 0)		ret_key = TRUE;
+    else if (Ustrcmp(ele, "cache=no") == 0)	cache = 0;
+    else if (Ustrcmp(ele, "cache=no_rd") == 0)	cache &= ~CACHE_RD;
+    else if (Ustrcmp(ele, "cache=no_wr") == 0)	cache &= ~CACHE_WR;
     else g = string_append_listele(g, ',', ele);
 
   opts = string_from_gstring(g);
@@ -768,43 +790,45 @@ if (opts)
 /* Arrange to put this database at the top of the LRU chain if it is a type
 that opens real files. */
 
-if (  open_top != (tree_node *)handle 
-   && lookup_list[t->name[0]-'0']->type == lookup_absfile)
+if (open_top != (tree_node *)handle)
   {
-  search_cache *c = (search_cache *)(t->data.ptr);
-  tree_node *up = c->up;
-  tree_node *down = c->down;
-
-  /* Cut it out of the list. A newly opened file will a NULL up pointer.
-  Otherwise there will be a non-NULL up pointer, since we checked above that
-  this block isn't already at the top of the list. */
-
-  if (up)
+  const lookup_info * li = lookup_with_acq_num(t->name[0]-'0');
+  if (li && li->type == lookup_absfile)
     {
-    ((search_cache *)(up->data.ptr))->down = down;
-    if (down)
-      ((search_cache *)(down->data.ptr))->up = up;
-    else
-      open_bot = up;
+    search_cache * c = (search_cache *)(t->data.ptr);
+    tree_node * up = c->up, * down = c->down;
+
+    /* Cut it out of the list. A newly opened file will have a NULL up pointer.
+    Otherwise there will be a non-NULL up pointer, since we checked above that
+    this block isn't already at the top of the list. */
+
+    if (up)
+      {
+      ((search_cache *)(up->data.ptr))->down = down;
+      if (down)
+	((search_cache *)(down->data.ptr))->up = up;
+      else
+	open_bot = up;
+      }
+
+    /* Now put it at the head of the list. */
+
+    c->up = NULL;
+    c->down = open_top;
+    if (!open_top) open_bot = t;
+    else ((search_cache *)(open_top->data.ptr))->up = t;
+    open_top = t;
     }
-
-  /* Now put it at the head of the list. */
-
-  c->up = NULL;
-  c->down = open_top;
-  if (!open_top) open_bot = t;
-  else ((search_cache *)(open_top->data.ptr))->up = t;
-  open_top = t;
   }
 
 DEBUG(D_lookup)
   {
   debug_printf_indent("LRU list:\n");
-  for (tree_node *t = open_top; t; )
+  for (tree_node * t = open_top; t; )
     {
     search_cache *c = (search_cache *)(t->data.ptr);
     debug_printf_indent("  %s\n", t->name);
-    if (t == open_bot) debug_printf_indent("  End\n");
+    if (t == open_bot) debug_printf_indent(" End\n");
     t = c->down;
     }
   }
@@ -812,7 +836,7 @@ DEBUG(D_lookup)
 /* First of all, try to match the key string verbatim. If matched a complete
 entry but could have been partial, flag to set up variables. */
 
-yield = internal_search_find(handle, filename, keystring, cache_rd, opts);
+yield = internal_search_find(handle, filename, keystring, cache, opts);
 if (f.search_find_defer) return NULL;
 
 if (yield) { if (partial >= 0) set_null_wild = TRUE; }
@@ -830,14 +854,16 @@ else if (partial >= 0)
 
   /* Try with the affix on the front, except for a zero-length affix */
 
-  if (affixlen == 0) keystring2 = keystring; else
+  if (affixlen == 0)
+    keystring2 = string_copy(keystring);
+  else
     {
     keystring2 = store_get(len + affixlen + 1,
 	  is_tainted(keystring) || is_tainted(affix) ? GET_TAINTED : GET_UNTAINTED);
     Ustrncpy(keystring2, affix, affixlen);
     Ustrcpy(keystring2 + affixlen, keystring);
     DEBUG(D_lookup) debug_printf_indent("trying partial match %s\n", keystring2);
-    yield = internal_search_find(handle, filename, keystring2, cache_rd, opts);
+    yield = internal_search_find(handle, filename, CUS keystring2, cache, opts);
     if (f.search_find_defer) return NULL;
     }
 
@@ -875,8 +901,8 @@ else if (partial >= 0)
         }
 
       DEBUG(D_lookup) debug_printf_indent("trying partial match %s\n", keystring3);
-      yield = internal_search_find(handle, filename, keystring3,
-		cache_rd, opts);
+      yield = internal_search_find(handle, filename, CUS keystring3,
+		cache, opts);
       if (f.search_find_defer) return NULL;
       if (yield)
         {
@@ -920,7 +946,7 @@ if (!yield  &&  starflags & SEARCH_STARAT)
     *atat = '*';
 
     DEBUG(D_lookup) debug_printf_indent("trying default match %s\n", atat);
-    yield = internal_search_find(handle, filename, atat, cache_rd, opts);
+    yield = internal_search_find(handle, filename, atat, cache, opts);
     *atat = savechar;
     if (f.search_find_defer) return NULL;
 
@@ -943,7 +969,7 @@ and the second is empty. */
 if (!yield  &&  starflags & (SEARCH_STAR|SEARCH_STARAT))
   {
   DEBUG(D_lookup) debug_printf_indent("trying to match *\n");
-  yield = internal_search_find(handle, filename, US"*", cache_rd, opts);
+  yield = internal_search_find(handle, filename, US"*", cache, opts);
   if (yield && expand_setup && *expand_setup >= 0)
     {
     *expand_setup += 1;
