@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -26,7 +26,7 @@ address (assuming HAVE_IPV6 is set). If a mask is permitted and one is present,
 and maskptr is not NULL, its offset is placed there.
 
 Arguments:
-  s         a string
+  ip_addr   a string
   maskptr   NULL if no mask is permitted to follow
             otherwise, points to an int where the offset of '/' is placed
             if there is no / followed by trailing digits, *maskptr is set 0
@@ -44,9 +44,9 @@ The legacy string_is_ip_address() function follows below.
 int
 string_is_ip_addressX(const uschar * ip_addr, int * maskptr, const uschar ** errp)
 {
-uschar * slash, * percent, * endp = NULL;
+uschar * slash, * percent;
 long int mask = 0;
-const uschar * addr = NULL;
+const uschar * endp = NULL, * addr = NULL;
 int af;
 union { /* we do not need this, but inet_pton() needs a place for storage */
   struct in_addr sa4;
@@ -111,7 +111,7 @@ if (endp)
   ptrdiff_t l = endp - ip_addr;
   if (l > 255)
     {
-    if (errp) *errp = US"rudiculous long ip address string";
+    if (errp) *errp = US"rediculous long ip address string";
     return 0;
     }
   addr = string_copyn(ip_addr, l);
@@ -269,7 +269,7 @@ int
 string_interpret_escape(const uschar **pp)
 {
 #ifdef COMPILE_UTILITY
-const uschar *hex_digits= CUS"0123456789abcdef";
+const uschar * hex_digits= CUS"0123456789abcdef";
 #endif
 int ch;
 const uschar *p = *pp;
@@ -322,13 +322,15 @@ macro string_printing(), which sets flags to 0.
 
 Arguments:
   s             the input string
-  flags		Bit 0: convert tabs.  Bit 1: convert spaces.
+  flags		Bit 0: convert tabs.
+		Bit 1: convert spaces.
+		Bit 2: convert doublequotes.
 
 Returns:        string with non-printers encoded as printing sequences
 */
 
 const uschar *
-string_printing2(const uschar *s, int flags)
+string_printing2(const uschar * s, int flags)
 {
 int nonprintcount = 0;
 int length = 0;
@@ -341,6 +343,7 @@ while (*t)
   if (  !mac_isprint(c)
      || flags & SP_TAB && c == '\t'
      || flags & SP_SPACE && c == ' '
+     || flags & SP_DQUOTES && c == '"'
      ) nonprintcount++;
   length++;
   }
@@ -360,6 +363,7 @@ for (t = s; *t; )
   if (  mac_isprint(c)
      && (!(flags & SP_TAB) || c != '\t')
      && (!(flags & SP_SPACE) || c != ' ')
+     && (!(flags & SP_DQUOTES) || c != '"')
      )
     *tt++ = *t++;
   else
@@ -373,6 +377,7 @@ for (t = s; *t; )
       case '\v': *tt++ = 'v'; break;
       case '\f': *tt++ = 'f'; break;
       case '\t': *tt++ = 't'; break;
+      case '"':  *tt++ = '"'; break;
       default: sprintf(CS tt, "%03o", *t); tt += 3; break;
       }
     t++;
@@ -399,10 +404,10 @@ Returns:        string with printing escapes parsed back
 */
 
 uschar *
-string_unprinting(uschar *s)
+string_unprinting(uschar * s)
 {
-uschar *p, *q, *r, *ss;
-int len, off;
+uschar * p, * q, * r, * ss;
+int len, offset;
 
 p = Ustrchr(s, '\\');
 if (!p) return s;
@@ -411,11 +416,10 @@ len = Ustrlen(s) + 1;
 ss = store_get(len, s);
 
 q = ss;
-off = p - s;
-if (off)
+if ((offset = p - s))
   {
-  memcpy(q, s, off);
-  q += off;
+  memcpy(q, s, offset);
+  q += offset;
   }
 
 while (*p)
@@ -430,17 +434,17 @@ while (*p)
     r = Ustrchr(p, '\\');
     if (!r)
       {
-      off = Ustrlen(p);
-      memcpy(q, p, off);
-      p += off;
-      q += off;
+      offset = Ustrlen(p);
+      memcpy(q, p, offset);
+      p += offset;
+      q += offset;
       break;
       }
     else
       {
-      off = r - p;
-      memcpy(q, p, off);
-      q += off;
+      offset = r - p;
+      memcpy(q, p, offset);
+      q += offset;
       p = r;
       }
     }
@@ -728,7 +732,7 @@ g = string_vformat_trc(g, func, line, STRING_SPRINTF_BUFFER_SIZE,
 va_end(ap);
 
 if (!g)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+  log_write_die(0, LOG_MAIN,
     "string_sprintf expansion was longer than %d; format string was (%s)\n"
     " called from %s %d\n",
     STRING_SPRINTF_BUFFER_SIZE, format, func, line);
@@ -842,8 +846,12 @@ while (*s)
 return NULL;
 }
 
+/*XXX C11 apparently has "generic functions", which allow the tracking
+of a parameter's type through to the return type.  Thit would neatly
+permit a single function name to be used, with better dev-safety, for this. */
+
 uschar *
-strstric(uschar * s, uschar * t, BOOL space_follows)
+strstric(const uschar * s, const uschar * t, BOOL space_follows)
 {
 return US strstric_c(s, t, space_follows);
 }
@@ -852,7 +860,7 @@ return US strstric_c(s, t, space_follows);
 #ifdef COMPILE_UTILITY
 /* Dummy version for this function; it should never be called */
 static void
-gstring_grow(gstring * g, int count)
+gstring_grow(const gstring * g, int count)
 {
 assert(FALSE);
 }
@@ -923,14 +931,18 @@ to be conservative. */
 
 while (isspace(*s) && *s != sep) s++;
 
-/* A change of separator is permitted, so look for a leading '<' followed by an
-allowed character. */
+/* A change of separator is permitted (assuming untainted source),
+so look for a leading '<' followed by an allowed character. */
 
 if (sep <= 0)
   {
   if (*s == '<' && (ispunct(s[1]) || iscntrl(s[1])))
     {
-    sep = s[1];
+    if (!is_tainted(s))
+      sep = s[1];
+    else DEBUG(D_any) 
+      debug_printf("attempt to use tainted change-of-seperator spec (%s %d)\n",
+		    config_filename, config_lineno);
     if (*++s) ++s;
     while (isspace(*s) && *s != sep) s++;
     }
@@ -948,7 +960,7 @@ if (!*s) return NULL;
 sep_is_special = iscntrl(sep);
 
 /* Handle the case when a buffer is provided. */
-/*XXX need to also deal with qouted-requirements mismatch */
+/*XXX need to also deal with quoted-requirements mismatch */
 
 if (buffer)
   {
@@ -957,7 +969,7 @@ if (buffer)
     die_tainted(US"string_nextinlist", func, line);
   for (; *s; s++)
     {
-    if (*s == sep && (*(++s) != sep || sep_is_special)) break;
+    if (*s == sep && (*++s != sep || sep_is_special)) break;
     if (p < buflen - 1) buffer[p++] = *s;
     }
   while (p > 0 && isspace(buffer[p-1])) p--;
@@ -1092,6 +1104,49 @@ return list;
 
 
 
+/* Listmaker that takes a format string and args for the element.
+A flag arg is required to handle embedded sep chars in the (expanded) element;
+if false then no check is done */
+
+gstring *
+string_append_listele_fmt(gstring * list, uschar sep, BOOL check,
+  const char * fmt, ...)
+{
+va_list ap;
+unsigned start;
+const gstring * g;
+
+if (list && list->ptr)
+  {
+  list = string_catn(list, &sep, 1);
+  start = list->ptr;
+  }
+else
+  start = 0;
+
+va_start(ap, fmt);
+list = string_vformat_trc(list, US __FUNCTION__, __LINE__,
+	  STRING_SPRINTF_BUFFER_SIZE, SVFMT_REBUFFER|SVFMT_EXTEND, fmt, ap);
+va_end(ap);
+
+(void) string_from_gstring(list);
+
+/* if the appended element turns out to have an embedded sep char, rewind
+and do the lazy-coded separate string method */
+
+if (!check || !Ustrchr(&list->s[start], sep))
+  return list;
+
+va_start(ap, fmt);
+g = string_vformat_trc(NULL, US __FUNCTION__, __LINE__,
+	STRING_SPRINTF_BUFFER_SIZE, SVFMT_REBUFFER|SVFMT_EXTEND, fmt, ap);
+va_end(ap);
+
+list->ptr = start;
+return string_append_listele_n(list, sep, g->s, g->ptr);
+}
+
+
 /* A slightly-bogus listmaker utility; the separator is a string so
 can be multiple chars - there is no checking for the element content
 containing any of the separator. */
@@ -1135,13 +1190,13 @@ existing length of the string. */
 unsigned inc = oldsize < 4096 ? 127 : 1023;
 
 if (g->ptr < 0 || g->ptr > g->size || g->size >= INT_MAX/2)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+  log_write_die(0, LOG_MAIN,
     "internal error in gstring_grow (ptr %d size %d)", g->ptr, g->size);
 
 if (count <= 0) return;
 
 if (count >= INT_MAX/2 - g->ptr)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+  log_write_die(0, LOG_MAIN,
     "internal error in gstring_grow (ptr %d count %d)", g->ptr, count);
 
 g->size = (p + count + inc + 1) & ~inc;		/* one for a NUL */
@@ -1192,7 +1247,7 @@ string_catn(gstring * g, const uschar * s, int count)
 int p;
 
 if (count < 0)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+  log_write_die(0, LOG_MAIN,
     "internal error in string_catn (count %d)", count);
 if (count == 0) return g;
 
@@ -1217,7 +1272,7 @@ else if (is_incompatible(g->s, s))
   }
 
 if (g->ptr < 0 || g->ptr > g->size)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+  log_write_die(0, LOG_MAIN,
     "internal error in string_catn (ptr %d size %d)", g->ptr, g->size);
 
 p = g->ptr;
@@ -1258,11 +1313,7 @@ string_append(gstring * g, int count, ...)
 va_list ap;
 
 va_start(ap, count);
-while (count-- > 0)
-  {
-  uschar * t = va_arg(ap, uschar *);
-  g = string_cat(g, t);
-  }
+while (count-- > 0) g = string_cat(g, va_arg(ap, uschar *));
 va_end(ap);
 
 return g;
@@ -1338,11 +1389,13 @@ If the "extend" flag is false, the string passed in may not be NULL,
 will not be grown, and is usable in the original place after return.
 The return value can be NULL to signify overflow.
 
+Alternate-form:		#: s/Y/b are silent about a null string
+			   H uppercase result
+Left-alignment:		-: only for s D H M S T Y V W Z b
 Field width:		decimal digits, or *
 Precision:		dot, followed by decimal digits or *
 Length modifiers:	h  L  l  ll  z
-Conversion specifiers:	n d o u x X p f e E g G % c s S T W V Y D M
-Alternate-form:		%#s is silent about a null string
+Conversion specifiers:	n d o u x X p f e E g G % c s S T W V Y D M H Z b q
 
 Returns the possibly-new (if copy for growth or taint-handling was needed)
 string, not nul-terminated.
@@ -1354,7 +1407,7 @@ string_vformat_trc(gstring * g, const uschar * func, unsigned line,
 {
 enum ltypes { L_NORMAL=1, L_SHORT=2, L_LONG=3, L_LONGLONG=4, L_LONGDOUBLE=5, L_SIZE=6 };
 
-int width, precision, off, lim, need;
+int width, precision, initial_off, lim, need;
 const char * fp = format;	/* Deliberately not unsigned */
 
 string_datestamp_offset = -1;	/* Datestamp not inserted */
@@ -1381,17 +1434,16 @@ if (!(flags & SVFMT_TAINT_NOCHK) && is_incompatible(g->s, format))
 #endif	/*!COMPILE_UTILITY*/
 
 lim = g->size - 1;	/* leave one for a nul */
-off = g->ptr;		/* remember initial offset in gstring */
+initial_off = g->ptr;	/* remember initial offset in gstring */
 
 /* Scan the format and handle the insertions */
 
 while (*fp)
   {
-  int length = L_NORMAL;
-  int * nptr;
-  int slen;
-  const char *null = "NULL";		/* ) These variables */
-  const char *item_start, *s;		/* ) are deliberately */
+  int length = L_NORMAL, slen, * nptr;
+  BOOL l_align = FALSE;
+  const char * null = "NULL";		/* ) These variables */
+  const char * item_start, * s;		/* ) are deliberately */
   char newformat[16];			/* ) not unsigned */
   char * gp = CS g->s + g->ptr;		/* ) */
 
@@ -1416,10 +1468,10 @@ while (*fp)
   item_start = fp;
   width = precision = -1;
 
-  if (strchr("-+ #0", *(++fp)) != NULL)
+  while (strchr("-+ #0", *++fp) != NULL)
     {
     if (*fp == '#') null = "";
-    fp++;
+    else if (*fp == '-') l_align = TRUE;
     }
 
   if (isdigit((uschar)*fp))
@@ -1463,7 +1515,7 @@ while (*fp)
     {
     case 'n':
       nptr = va_arg(ap, int *);
-      *nptr = g->ptr - off;
+      *nptr = g->ptr - initial_off;
       break;
 
     case 'd':
@@ -1501,7 +1553,7 @@ while (*fp)
 
     case 'p':
       {
-      void * ptr;
+      const void * ptr;
       if ((need = g->ptr + 24) > lim)
 	{
 	if (!(flags & SVFMT_EXTEND || need >= size_limit)) return NULL;
@@ -1552,7 +1604,7 @@ while (*fp)
 
     /* String types */
 
-    case '%':
+    case '%':			/* a literal '%' */
       if ((need = g->ptr + 1) > lim)
 	{
 	if (!(flags & SVFMT_EXTEND || need >= size_limit)) return NULL;
@@ -1562,7 +1614,7 @@ while (*fp)
       g->s[g->ptr++] = (uschar) '%';
       break;
 
-    case 'c':
+    case 'c':			/* a char */
       if ((need = g->ptr + 1) > lim)
 	{
 	if (!(flags & SVFMT_EXTEND || need >= size_limit)) return NULL;
@@ -1596,7 +1648,15 @@ while (*fp)
       goto INSERT_GSTRING;
       }
 #ifndef COMPILE_UTILITY
-    case 'V':			/* Maybe convert ascii-art to UTF-8 chars */
+    case 'b':			/* blob pointer, carrying a string */
+      {
+      blob * b = va_arg(ap, blob *);
+      if (b) { s = CS b->data; slen = b->len; }
+      else   { s = null;       slen = Ustrlen(s); }
+      goto INSERT_GSTRING;
+      }
+
+    case 'V':		/* string; maybe convert ascii-art to UTF-8 chars */
       {
       gstring * zg = NULL;
       s = va_arg(ap, char *);
@@ -1626,17 +1686,18 @@ while (*fp)
 
     case 'W':			/* Maybe mark up ctrls, spaces & newlines */
       s = va_arg(ap, char *);
-      if (Ustrpbrk(s, " \n") && !IS_DEBUG(D_noutf8))
+      if (s && !IS_DEBUG(D_noutf8))
 	{
 	gstring * zg = NULL;
 	int p = precision;
-	for ( ; *s; s++)
-	  {
-	  /* Take a given precision as applying to the input; expand
-	  it for the transformed result */
 
-	  if (p >= 0 && --p < 0) break;
-	  switch (*s)
+	/* If a precision was given, we can handle embedded NULs. Take it as
+	applying to the input and expand it for the transformed result */
+
+	for ( ; precision >= 0 || *s; s++)
+	  if (p >= 0 && --p < 0)
+	    break;
+	  else switch (*s)
 	    {
 	    case ' ':
 	      zg = string_catn(zg, CUS UTF8_LIGHT_SHADE, 3);
@@ -1658,9 +1719,8 @@ while (*fp)
 		zg = string_catn(zg, CUS s, 1);
 	      break;
 	    }
-	  }
 	if (zg) { s = CS zg->s; slen = gstring_length(zg); }
-	else    { s = null;     slen = Ustrlen(s); }
+	else    { s = "";	slen = 0; }
 	}
       else
 	{
@@ -1669,6 +1729,72 @@ while (*fp)
 	}
       goto INSERT_GSTRING;
 
+    case 'Z':			/* pdkim-style "quoteprint" */
+      {
+      gstring * zg = NULL;
+      int p = precision;	/* If given, we can handle embedded NULs */
+
+      s = va_arg(ap, char *);
+      for ( ; precision >= 0 || *s; s++)
+	if (p >= 0 && --p < 0)
+	  break;
+	else switch (*s)
+	  {
+	  case ' ' : zg = string_catn(zg, US"{SP}", 4); break;
+	  case '\t': zg = string_catn(zg, US"{TB}", 4); break;
+	  case '\r': zg = string_catn(zg, US"{CR}", 4); break;
+	  case '\n': zg = string_catn(zg, US"{LF}", 4); break;
+	  case '{' : zg = string_catn(zg, US"{BO}", 4); break;
+	  case '}' : zg = string_catn(zg, US"{BC}", 4); break;
+	  default:
+	    {
+	    uschar u = *s;
+	    if ( (u < 32) || (u > 127) )
+	      zg = string_fmt_append(zg, "{%02x}", u);
+	    else
+	      zg = string_catn(zg, US s, 1);
+	    break;
+	    }
+	  }
+      if (zg) { s = CS zg->s; precision = slen = gstring_length(zg); }
+      else    { s = "";	slen = 0; }
+      goto INSERT_GSTRING;
+      }
+
+    case 'H':			/* pdkim-style "hexprint" */
+      {
+      s = va_arg(ap, char *);
+      if (precision < 0) break;	/* precision must be given; src byte count */
+      if (s)
+	{
+	gstring * zg = NULL;
+	width -= 2;	/* min field width for each i/p byte. Spaces left-pad */
+	for (int p = precision; p > 0; p--)
+	  {
+	  for (int w = width; w > 0; w--) zg = string_catn(zg, US" ", 1);
+	  zg = string_fmt_append(zg, *null ? "%02x" : "%02X", * US s++);
+	  }
+
+	if (zg) { s = CS zg->s; precision = slen = gstring_length(zg); }
+	else    { s = "";	slen = 0; }
+	}
+      else
+	{ s = "<NULL>"; precision = slen = 6; }
+      }
+      goto INSERT_GSTRING;
+
+    case 'q':			/* string, to be wrapped in "" and with tab & " escaped */
+      if ((s = va_arg(ap, char *)))
+	{
+	gstring * zg = string_catn(NULL, US"\"", 1);
+	zg = string_cat(zg, string_printing2(US s, SP_TAB | SP_DQUOTES));
+	zg = string_catn(zg, US"\"", 1);
+	s = CS zg->s; precision = slen = gstring_length(zg);
+	}
+      else
+	{ s = "<NULL>"; precision = slen = 6; }
+      goto INSERT_GSTRING;
+
 #endif
     case 's':
     case 'S':                   /* Forces *lower* case */
@@ -1676,7 +1802,7 @@ while (*fp)
       s = va_arg(ap, char *);
 
       if (!s) s = null;
-      slen = Ustrlen(s);
+      slen = precision < 0 ? Ustrlen(s) : strnlen(CCS s, precision);
 
     INSERT_GSTRING:		/* Come to from %Y above */
 
@@ -1735,7 +1861,8 @@ while (*fp)
 	gp = CS g->s + g->ptr;
 	}
 
-      g->ptr += sprintf(gp, "%*.*s", width, precision, s);
+      g->ptr += sprintf(gp, l_align ? "%-*.*s" : "%*.*s",
+	width, precision, s);
       if (fp[-1] == 'S')
 	while (*gp) { *gp = tolower(*gp); gp++; }
       else if (fp[-1] == 'T')
@@ -1750,14 +1877,14 @@ while (*fp)
     default:
       strncpy(newformat, item_start, fp - item_start);
       newformat[fp-item_start] = 0;
-      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "string_format: unsupported type "
-	"in \"%s\" in \"%s\"", newformat, format);
+      log_write_die(0, LOG_MAIN, "string_format: unsupported type "
+	"in %q in %q", newformat, format);
       break;
     }
   }
 
 if (g->ptr > g->size)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+  log_write_die(0, LOG_MAIN,
     "string_format internal error: caller %s %d", func, line);
 return g;
 }
