@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -44,7 +44,7 @@ optionlist optionlist_transports[] = {
   { "disable_logging",  opt_bool|opt_public,
                  LOFF(disable_logging) },
   { "driver",           opt_stringptr|opt_public,
-                 LOFF(driver_name) },
+                 LOFF(drinst.driver_name) },
   { "envelope_to_add",   opt_bool|opt_public,
                  LOFF(envelope_to_add) },
 #ifndef DISABLE_EVENT
@@ -91,6 +91,44 @@ optionlist optionlist_transports[] = {
 
 int optionlist_transports_size = nelem(optionlist_transports);
 
+/******************************************************************************/
+/*
+ * Needed by both MACRO_PREDEF and the actual exim binary: build a list
+ * of transport drivers. The content differs for the two cases.
+ */
+static void
+transport_init_statics(transport_info ** anchor)
+{
+driver_info ** a = (driver_info **) anchor;
+
+#if defined(TRANSPORT_APPENDFILE) && (TRANSPORT_APPENDFILE!=2 || defined(MACRO_PREDEF))
+  extern transport_info appendfile_transport_info;
+  add_driver_info(a, &appendfile_transport_info.drinfo, sizeof(transport_info));
+#endif
+#if defined(TRANSPORT_AUTOREPLY) && (TRANSPORT_AUTOREPLY!=2 || defined(MACRO_PREDEF))
+  extern transport_info autoreply_transport_info;
+  add_driver_info(a, &autoreply_transport_info.drinfo, sizeof(transport_info));
+#endif
+#if defined(TRANSPORT_LMTP) && (TRANSPORT_LMTP!=2 || defined(MACRO_PREDEF))
+  extern transport_info lmtp_transport_info;
+  add_driver_info(a, &lmtp_transport_info.drinfo, sizeof(transport_info));
+#endif
+#if defined(TRANSPORT_PIPE) && (TRANSPORT_PIPE!=2 || defined(MACRO_PREDEF))
+  extern transport_info pipe_transport_info;
+  add_driver_info(a, &pipe_transport_info.drinfo, sizeof(transport_info));
+#endif
+#if defined(EXPERIMENTAL_QUEUEFILE) && (EXPERIMENTAL_QUEUEFILE!=2 || defined(MACRO_PREDEF))
+  extern transport_info queuefile_transport_info;
+  add_driver_info(a, &queuefile_transport_info.drinfo, sizeof(transport_info));
+#endif
+#if defined(TRANSPORT_SMTP) && (TRANSPORT_SMTP!=2 || defined(MACRO_PREDEF))
+  extern transport_info smtp_transport_info;
+  add_driver_info(a, &smtp_transport_info.drinfo, sizeof(transport_info));
+#endif
+}
+
+/******************************************************************************/
+
 #ifdef MACRO_PREDEF
 
 # include "macro_predef.h"
@@ -98,15 +136,18 @@ int optionlist_transports_size = nelem(optionlist_transports);
 void
 options_transports(void)
 {
-uschar buf[64];
-
 options_from_list(optionlist_transports, nelem(optionlist_transports), US"TRANSPORTS", NULL);
 
-for (transport_info * ti = transports_available; ti->driver_name[0]; ti++)
+transport_init_statics(&transports_available);
+
+for (driver_info * di= (driver_info *)transports_available; di; di = di->next)
   {
-  spf(buf, sizeof(buf), US"_DRIVER_TRANSPORT_%T", ti->driver_name);
+  uschar buf[64];
+
+  spf(buf, sizeof(buf), US"_DRIVER_TRANSPORT_%T", di->driver_name);
   builtin_macro_create(buf);
-  options_from_list(ti->options, (unsigned)*ti->options_count, US"TRANSPORT", ti->driver_name);
+  options_from_list(di->options, (unsigned)*di->options_count,
+		    US"TRANSPORT", di->driver_name);
   }
 }
 
@@ -144,28 +185,43 @@ the work. */
 void
 transport_init(void)
 {
-readconf_driver_init(US"transport",
-  (driver_instance **)(&transports),     /* chain anchor */
-  (driver_info *)transports_available,   /* available drivers */
-  sizeof(transport_info),                /* size of info block */
-  &transport_defaults,                   /* default values for generic options */
-  sizeof(transport_instance),            /* size of instance block */
-  optionlist_transports,                 /* generic options */
-  optionlist_transports_size);
+int old_pool = store_pool;
+store_pool = POOL_PERM;
+  {
+  /* Add the transport drivers that are built for static linkage to the
+  list of availables. */
+
+  transport_init_statics(&transports_available);
+  }
+store_pool = old_pool;
+
+/* Read the config file "transports" section, creating a transports instance list.
+For any yet-undiscovered driver, check for a loadable module and add it to
+those available. */
+
+readconf_driver_init((driver_instance **)&transports,     /* chain anchor */
+  (driver_info **)&transports_available, /* available drivers */
+  sizeof(transport_info),		/* size of info block */
+  &transport_defaults,			/* default values for generic options */
+  sizeof(transport_instance),		/* size of instance block */
+  optionlist_transports,		/* generic options */
+  optionlist_transports_size,
+  US"transport");
 
 /* Now scan the configured transports and check inconsistencies. A shadow
 transport is permitted only for local transports. */
 
-for (transport_instance * t = transports; t; t = t->next)
+for (transport_instance * t = transports; t; t = t->drinst.next)
   {
-  if (!t->info->local && t->shadow)
-    log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
-      "shadow transport not allowed on non-local transport %s", t->name);
+  const transport_info * ti = t->drinst.info;
+  if (!ti->local && t->shadow)
+    log_write_die(0, LOG_CONFIG,
+      "shadow transport not allowed on non-local transport %s", t->drinst.name);
 
   if (t->body_only && t->headers_only)
-    log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
+    log_write_die(0, LOG_CONFIG,
       "%s transport: body_only and headers_only are mutually exclusive",
-      t->name);
+      t->drinst.name);
   }
 }
 
@@ -352,13 +408,21 @@ transport_write_block(transport_ctx * tctx, uschar *block, int len, BOOL more)
 {
 if (!(tctx->options & topt_output_string))
   return transport_write_block_fd(tctx, block, len, more);
+else
+  {
+  int old_pool = store_pool;
 
-/* Write to expanding-string.  NOTE: not NUL-terminated */
+  /* Write to expanding-string.  NOTE: not NUL-terminated.
+  Use perm-pool as our callers may be doing mark/release and we need
+  the stored string later. */
 
-if (!tctx->u.msg)
-  tctx->u.msg = string_get(1024);
+  store_pool = POOL_PERM;
+  if (!tctx->u.msg)
+    tctx->u.msg = string_get(1024);
 
-tctx->u.msg = string_catn(tctx->u.msg, block, len);
+  tctx->u.msg = string_catn(tctx->u.msg, block, len);
+  store_pool = old_pool;
+  }
 return TRUE;
 }
 
@@ -391,7 +455,7 @@ that the result will never be expanded. */
 
 va_start(ap, format);
 if (!string_vformat(&gs, SVFMT_TAINT_NOCHK, format, ap))
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "overlong formatted string in transport");
+  log_write_die(0, LOG_MAIN, "overlong formatted string in transport");
 va_end(ap);
 tctx.u.fd = fd;
 return transport_write_block(&tctx, gs.s, gs.ptr, FALSE);
@@ -426,7 +490,7 @@ Arguments:
   tctx       transport context - processing to be done during output,
 		and file descriptor to write to
   chunk      pointer to data to write
-  len        length of data to write
+  dlen       length of data to write
 
 In addition, the static nl_xxx variables must be set as required.
 
@@ -434,10 +498,9 @@ Returns:     TRUE on success, FALSE on failure (with errno preserved)
 */
 
 BOOL
-write_chunk(transport_ctx * tctx, const uschar * chunk, int len)
+write_chunk(transport_ctx * tctx, const uschar * chunk, int dlen)
 {
-const uschar * start = chunk;
-const uschar * end = chunk + len;
+const uschar * start = chunk, * end = chunk + dlen;
 int mlen = DELIVER_OUT_BUFFER_SIZE - nl_escape_length - 2;
 
 /* The assumption is made that the check string will never stretch over move
@@ -449,7 +512,7 @@ match. */
 
 if (nl_partial_match >= 0)
   {
-  if (nl_check_length > 0 && len >= nl_check_length &&
+  if (nl_check_length > 0 && dlen >= nl_check_length &&
       Ustrncmp(start, nl_check + nl_partial_match,
         nl_check_length - nl_partial_match) == 0)
     {
@@ -476,13 +539,14 @@ possible. */
 
 for (const uschar * ptr = start; ptr < end; ptr++)
   {
-  int ch, len;
+  int fl_len;
+  uschar ch;
 
   /* Flush the buffer if it has reached the threshold - we want to leave enough
   room for the next uschar, plus a possible extra CR for an LF, plus the escape
   string. */
 
-  if ((len = chunk_ptr - deliver_out_buffer) > mlen)
+  if ((fl_len = chunk_ptr - deliver_out_buffer) > mlen)
     {
     DEBUG(D_transport) debug_printf("flushing headers buffer\n");
 
@@ -491,14 +555,14 @@ for (const uschar * ptr = start; ptr < end; ptr++)
 
     if (tctx->options & topt_use_bdat  &&  tctx->chunk_cb)
       {
-      if (  tctx->chunk_cb(tctx, (unsigned)len, 0) != OK
-	 || !transport_write_block(tctx, deliver_out_buffer, len, FALSE)
+      if (  tctx->chunk_cb(tctx, (unsigned)fl_len, 0) != OK
+	 || !transport_write_block(tctx, deliver_out_buffer, fl_len, FALSE)
 	 || tctx->chunk_cb(tctx, 0, tc_reap_prev) != OK
 	 )
 	return FALSE;
       }
     else
-      if (!transport_write_block(tctx, deliver_out_buffer, len, FALSE))
+      if (!transport_write_block(tctx, deliver_out_buffer, fl_len, FALSE))
 	return FALSE;
     chunk_ptr = deliver_out_buffer;
     }
@@ -964,7 +1028,7 @@ if (!(tctx->options & topt_no_headers))
   if (tctx->options & topt_add_return_path)
     {
     int n;
-    uschar * s = string_sprintf("Return-path: <%.*s>\n%n",
+    const uschar * s = string_sprintf("Return-path: <%.*s>\n%n",
                           EXIM_EMAILADDR_MAX, return_path, &n);
     if (!write_chunk(tctx, s, n)) goto bad;
     }
@@ -998,7 +1062,7 @@ if (!(tctx->options & topt_no_headers))
 
   if (tctx->options & topt_add_delivery_date)
     {
-    uschar * s = tod_stamp(tod_full);
+    const uschar * s = tod_stamp(tod_full);
 
     if (  !write_chunk(tctx, US"Delivery-date: ", 15)
        || !write_chunk(tctx, s, Ustrlen(s))
@@ -1028,7 +1092,7 @@ flush that buffer with non-last BDAT prependix) plus the amount of body data
 (as expanded for CRLF lines).  Then create and write BDAT(s), and ensure
 that further use of write_chunk() will not prepend BDATs.
 The first BDAT written will also first flush any outstanding MAIL and RCPT
-commands which were buffered thans to PIPELINING.
+commands which were buffered thanks to PIPELINING.
 Commands go out (using a send()) from a different buffer to data (using a
 write()).  They might not end up in the same TCP segment, which is
 suboptimal. */
@@ -1135,18 +1199,18 @@ DEBUG(D_transport)
 
 if (!(tctx->options & topt_no_body))
   {
-  unsigned long size = size_limit > 0 ? size_limit : ULONG_MAX;
+  unsigned long bsize = size_limit > 0 ? size_limit : ULONG_MAX;
 
   nl_check_length = abs(nl_check_length);
   nl_partial_match = 0;
   if (lseek(deliver_datafile, spool_data_start_offset(message_id), SEEK_SET) < 0)
     return FALSE;
-  while (  (len = MIN(DELIVER_IN_BUFFER_SIZE, size)) > 0
+  while (  (len = MIN(DELIVER_IN_BUFFER_SIZE, bsize)) > 0
 	&& (len = read(deliver_datafile, deliver_in_buffer, len)) > 0)
     {
     if (!write_chunk(tctx, deliver_in_buffer, len))
       return FALSE;
-    size -= len;
+    bsize -= len;
     }
 
   /* A read error on the body will have left len == -1 and errno set. */
@@ -1163,7 +1227,7 @@ f.spool_file_wireformat = FALSE;
 
 if (tctx->options & topt_end_dot)
   {
-  smtp_debug_cmd(US".", 0);
+  smtp_debug_cmd(US".", tctx->options & topt_no_flush ? SCMD_MORE: 0);
   if (!write_chunk(tctx, US".\n", 2))
     return FALSE;
   }
@@ -1263,7 +1327,7 @@ smtp dots, or check string processing. */
 if (pipe(pfd) != 0) goto TIDY_UP;      /* errno set */
 if ((write_pid = exim_fork(US"tpt-filter-writer")) == 0)
   {
-  BOOL rc;
+  BOOL written_ok;
   (void)close(fd_read);
   (void)close(pfd[pipe_read]);
   nl_check_length = nl_escape_length = 0;
@@ -1272,10 +1336,10 @@ if ((write_pid = exim_fork(US"tpt-filter-writer")) == 0)
   tctx->check_string = tctx->escape_string = NULL;
   tctx->options &= ~(topt_use_crlf | topt_end_dot | topt_use_bdat | topt_no_flush);
 
-  rc = internal_transport_write_message(tctx, size_limit);
+  written_ok = internal_transport_write_message(tctx, size_limit);
 
   save_errno = errno;
-  if (  write(pfd[pipe_write], (void *)&rc, sizeof(BOOL))
+  if (  write(pfd[pipe_write], (void *)&written_ok, sizeof(BOOL))
         != sizeof(BOOL)
      || write(pfd[pipe_write], (void *)&save_errno, sizeof(int))
         != sizeof(int)
@@ -1284,7 +1348,7 @@ if ((write_pid = exim_fork(US"tpt-filter-writer")) == 0)
      || write(pfd[pipe_write], (void *)&tctx->addr->delivery_time, sizeof(struct timeval))
         != sizeof(struct timeval)
      )
-    rc = FALSE;	/* compiler quietening */
+    written_ok = FALSE;	/* compiler quietening */
   exim_underbar_exit(EXIT_SUCCESS);
   }
 save_errno = errno;
@@ -1410,6 +1474,7 @@ if (write_pid > 0)
 	int dummy = read(pfd[pipe_read], (void *)&save_errno, sizeof(int));
         dummy = read(pfd[pipe_read], (void *)&tctx->addr->more_errno, sizeof(int));
         dummy = read(pfd[pipe_read], (void *)&tctx->addr->delivery_time, sizeof(struct timeval));
+        dummy = dummy;	/* (more) compiler quietening */
         yield = FALSE;
         }
       }
@@ -1497,11 +1562,10 @@ Returns:    nothing
 */
 
 void
-transport_update_waiting(host_item * hostlist, uschar * tpname)
+transport_update_waiting(host_item * hostlist, const uschar * tpname)
 {
-const uschar *prevname = US"";
-open_db dbblock;
-open_db *dbm_file;
+const uschar * prevname = US"";
+open_db dbblock, * dbp;
 
 if (!is_new_message_id(message_id))
   {
@@ -1510,13 +1574,17 @@ if (!is_new_message_id(message_id))
   return;
   }
 
-DEBUG(D_transport) debug_printf("updating wait-%s database\n", tpname);
+DEBUG(D_transport)
+  { debug_printf("updating wait-%s database\n", tpname); acl_level++; }
 
-/* Open the database for this transport */
+/* Open the database (or transaction) for this transport */
 
-if (!(dbm_file = dbfn_open(string_sprintf("wait-%.200s", tpname),
-		      O_RDWR, &dbblock, TRUE, TRUE)))
-  return;
+if ( continue_wait_db
+   ? !dbfn_transaction_start(dbp = continue_wait_db)
+   : !(dbp = dbfn_open(string_sprintf("wait-%.200s", tpname),
+		      O_RDWR|O_CREAT, &dbblock, TRUE, TRUE))
+   )
+  goto out;
 
 /* Scan the list of hosts for which this message is waiting, and ensure
 that the message id is in each host record. */
@@ -1536,7 +1604,7 @@ for (host_item * host = hostlist; host; host = host->next)
 
   /* Look up the host record; if there isn't one, make an empty one. */
 
-  if (!(host_record = dbfn_read(dbm_file, host->name)))
+  if (!(host_record = dbfn_read(dbp, host->name)))
     {
     host_record = store_get(sizeof(dbdata_wait) + MESSAGE_ID_LENGTH, GET_UNTAINTED);
     host_record->count = host_record->sequence = 0;
@@ -1560,9 +1628,9 @@ for (host_item * host = hostlist; host; host = host->next)
 	debug_printf_indent("NOTE: old or corrupt message-id found in wait=%.200s"
 	  " hints DB; deleting records for %s\n", tpname, host->name);
 
-      (void) dbfn_delete(dbm_file, host->name);
+      (void) dbfn_delete(dbp, host->name);
       for (int i = host_record->sequence - 1; i >= 0; i--)
-	(void) dbfn_delete(dbm_file,
+	(void) dbfn_delete(dbp,
 		    (sprintf(CS buffer, "%.200s:%d", host->name, i), buffer));
 
       host_record->count = host_record->sequence = 0;
@@ -1579,7 +1647,7 @@ for (host_item * host = hostlist; host; host = host->next)
     {
     dbdata_wait *cont;
     sprintf(CS buffer, "%.200s:%d", host->name, i);
-    if ((cont = dbfn_read(dbm_file, buffer)))
+    if ((cont = dbfn_read(dbp, buffer)))
       {
       int clen = cont->count * MESSAGE_ID_LENGTH;
       for (uschar * s = cont->text; s < cont->text + clen; s += MESSAGE_ID_LENGTH)
@@ -1605,7 +1673,7 @@ for (host_item * host = hostlist; host; host = host->next)
   if (host_record->count >= WAIT_NAME_MAX)
     {
     sprintf(CS buffer, "%.200s:%d", host->name, host_record->sequence);
-    dbfn_write(dbm_file, buffer, host_record, sizeof(dbdata_wait) + host_length);
+    dbfn_write(dbp, buffer, host_record, sizeof(dbdata_wait) + host_length);
 #ifndef DISABLE_QUEUE_RAMP
     if (f.queue_2stage && queue_fast_ramp && !queue_run_in_order)
       queue_notify_daemon(message_id);
@@ -1634,14 +1702,21 @@ for (host_item * host = hostlist; host; host = host->next)
 
   /* Update the database */
 
-  dbfn_write(dbm_file, host->name, host_record, sizeof(dbdata_wait) + host_length);
+  dbfn_write(dbp, host->name, host_record, sizeof(dbdata_wait) + host_length);
   DEBUG(D_transport) debug_printf("added %.*s to queue for %s\n",
 				  MESSAGE_ID_LENGTH, message_id, host->name);
   }
 
 /* All now done */
 
-dbfn_close(dbm_file);
+if (continue_wait_db)
+  dbfn_transaction_commit(dbp);
+else
+  dbfn_close(dbp);
+
+out:
+  DEBUG(D_transport) acl_level--;
+  return;
 }
 
 
@@ -1688,8 +1763,7 @@ transport_check_waiting(const uschar * transport_name, const uschar * hostname,
 {
 dbdata_wait * host_record;
 int host_length;
-open_db dbblock;
-open_db * dbm_file;
+open_db dbblock, * dbp;
 
 int         i;
 struct stat statbuf;
@@ -1715,17 +1789,24 @@ if (local_message_max > 0 && continue_sequence >= local_message_max)
 
 /* Open the waiting information database. */
 
-if (!(dbm_file = dbfn_open(string_sprintf("wait-%.200s", transport_name),
-			  O_RDWR, &dbblock, TRUE, TRUE)))
+if ( continue_wait_db
+   ? !dbfn_transaction_start(dbp = continue_wait_db)
+   : !(dbp = dbfn_open(string_sprintf("wait-%.200s", transport_name),
+		      O_RDWR, &dbblock, TRUE, TRUE))
+   )
+  {
+  DEBUG(D_transport)
+    debug_printf_indent("no messages waiting for %s\n", hostname);
   goto retfalse;
+  }
 
 /* See if there is a record for this host; if not, there's nothing to do. */
 
-if (!(host_record = dbfn_read(dbm_file, hostname)))
+if (!(host_record = dbfn_read(dbp, hostname)))
   {
-  dbfn_close(dbm_file);
-  DEBUG(D_transport) debug_printf_indent("no messages waiting for %s\n", hostname);
-  goto retfalse;
+  DEBUG(D_transport)
+    debug_printf_indent("no messages waiting for %s\n", hostname);
+  goto dbclose_false;
   }
 
 /* If the data in the record looks corrupt, just log something and
@@ -1733,10 +1814,9 @@ don't try to use it. */
 
 if (host_record->count > WAIT_NAME_MAX)
   {
-  dbfn_close(dbm_file);
   log_write(0, LOG_MAIN|LOG_PANIC, "smtp-wait database entry for %s has bad "
     "count=%d (max=%d)", hostname, host_record->count, WAIT_NAME_MAX);
-  goto retfalse;
+  goto dbclose_false;
   }
 
 /* Scan the message ids in the record in order
@@ -1770,13 +1850,14 @@ while (1)
       {
       uschar buffer[256];
       DEBUG(D_hints_lookup)
-	debug_printf_indent("NOTE: old or corrupt message-id found in wait=%.200s"
-	  " hints DB; deleting records for %s\n", transport_name, hostname);
-      (void) dbfn_delete(dbm_file, hostname);
+	debug_printf_indent("NOTE: old or corrupt message-id found in"
+	  " wait=%.200s hints DB; deleting records for %s\n",
+	  transport_name, hostname);
+      (void) dbfn_delete(dbp, hostname);
       for (int j = host_record->sequence - 1; j >= 0; j--)
-	(void) dbfn_delete(dbm_file,
+	(void) dbfn_delete(dbp,
 		    (sprintf(CS buffer, "%.200s:%d", hostname, j), buffer));
-      goto retfalse;
+      goto dbclose_false;
       }
     msgq[i].bKeep = TRUE;
 
@@ -1791,6 +1872,8 @@ while (1)
   for (i = 0; i < msgq_count; ++i)
     if (Ustrcmp(msgq[i].message_id, message_id) == 0)
       {
+      DEBUG(D_hints_lookup)
+	debug_printf_indent("dropping current msg from list\n");
       msgq[i].bKeep = FALSE;
       break;
       }
@@ -1807,6 +1890,8 @@ while (1)
       msgq[i].bKeep = FALSE;
     else if (!oicf_func || oicf_func(mid, oicf_data))
       {
+      DEBUG(D_hints_lookup)
+	debug_printf_indent("acceptable next: %s\n", mid);
       Ustrcpy_nt(new_message_id, mid);
       msgq[i].bKeep = FALSE;
       bFound = TRUE;
@@ -1816,8 +1901,9 @@ while (1)
 
   /* re-count */
   for (msgq_actual = 0, i = 0; i < msgq_count; ++i)
-    if (msgq[i].bKeep)
-      msgq_actual++;
+    if (msgq[i].bKeep) msgq_actual++;
+  DEBUG(D_hints_lookup)
+    debug_printf_indent("%d left in this record\n", msgq_actual);
 
   /* reassemble the host record, based on removed message ids, from in
   memory queue  */
@@ -1853,23 +1939,23 @@ while (1)
 
     /* Search for a continuation */
 
-    for (int i = host_record->sequence - 1; i >= 0 && !newr; i--)
+    for (int j = host_record->sequence - 1; j >= 0 && !newr; j--)
       {
-      sprintf(CS buffer, "%.200s:%d", hostname, i);
-      newr = dbfn_read(dbm_file, buffer);
+      sprintf(CS buffer, "%.200s:%d", hostname, j);
+      newr = dbfn_read(dbp, buffer);
       }
 
     /* If no continuation, delete the current and break the loop */
 
     if (!newr)
       {
-      dbfn_delete(dbm_file, hostname);
+      dbfn_delete(dbp, hostname);
       break;
       }
 
     /* Else replace the current with the continuation */
 
-    dbfn_delete(dbm_file, buffer);
+    dbfn_delete(dbp, buffer);
     host_record = newr;
     host_length = host_record->count * MESSAGE_ID_LENGTH;
 
@@ -1885,20 +1971,19 @@ while (1)
 
   if (host_length <= 0)
     {
-    dbfn_close(dbm_file);
-    DEBUG(D_transport) debug_printf_indent("waiting messages already delivered\n");
-    goto retfalse;
+    DEBUG(D_transport|D_hints_lookup)
+      debug_printf_indent("waiting messages already delivered\n");
+    goto dbclose_false;
     }
 
   /* we were not able to find an acceptable message, nor was there a
-  continuation record.  So bug out, outer logic will clean this up.
-  */
+  continuation record.  So bug out, outer logic will clean this up. */
 
   if (!bContinuation)
     {
+    DEBUG(D_hints_lookup) debug_printf_indent("no further records\n");
     Ustrcpy(new_message_id, message_id);
-    dbfn_close(dbm_file);
-    goto retfalse;
+    goto dbclose_false;
     }
   }		/* we need to process a continuation record */
 
@@ -1910,10 +1995,14 @@ record if required, close the database, and return TRUE. */
 if (host_length > 0)
   {
   host_record->count = host_length/MESSAGE_ID_LENGTH;
-  dbfn_write(dbm_file, hostname, host_record, (int)sizeof(dbdata_wait) + host_length);
+  dbfn_write(dbp, hostname, host_record, (int)sizeof(dbdata_wait) + host_length);
   }
 
-dbfn_close(dbm_file);
+if (continue_wait_db)
+  dbfn_transaction_commit(dbp);
+else
+  dbfn_close(dbp);
+
 DEBUG(D_transport)
   {
   acl_level--;
@@ -1921,9 +2010,16 @@ DEBUG(D_transport)
   }
 return TRUE;
 
+dbclose_false:
+  if (continue_wait_db)
+    dbfn_transaction_commit(dbp);
+  else
+    dbfn_close(dbp);
+
 retfalse:
-DEBUG(D_transport) {acl_level--; debug_printf("transport_check_waiting: FALSE\n"); }
-return FALSE;
+  DEBUG(D_transport)
+    {acl_level--; debug_printf("transport_check_waiting: FALSE\n"); }
+  return FALSE;
 }
 
 /*************************************************
@@ -1933,9 +2029,9 @@ return FALSE;
 /* Just the regain-root-privilege exec portion */
 void
 transport_do_pass_socket(const uschar * transport_name, const uschar * hostname,
-  const uschar * hostaddress, uschar * id, int socket_fd)
+  const uschar * hostaddress, int hostport, uschar * id, int socket_fd)
 {
-int i = 13;
+int i = 14;
 const uschar **argv;
 
 #ifndef DISABLE_TLS
@@ -1984,7 +2080,7 @@ if (smtp_peer_options & OPTION_TLS)
 #endif
 
 #ifndef DISABLE_ESMTP_LIMITS
-if (continue_limit_rcpt || continue_limit_rcptdom)
+if (continue_limit_mail || continue_limit_rcpt || continue_limit_rcptdom)
   {
   argv[i++] = US"-MCL";
   argv[i++] = string_sprintf("%u", continue_limit_mail);
@@ -2015,6 +2111,7 @@ argv[i++] = US"-MC";
 argv[i++] = US transport_name;
 argv[i++] = US hostname;
 argv[i++] = US hostaddress;
+argv[i++] = string_sprintf("%d", hostport);
 argv[i++] = string_sprintf("%d", continue_sequence + 1);
 argv[i++] = id;
 argv[i++] = NULL;
@@ -2038,73 +2135,6 @@ _exit(errno);         /* Note: must be _exit(), NOT exit() */
 
 
 
-/* Fork a new exim process to deliver the message, and do a re-exec, both to
-get a clean delivery process, and to regain root privilege in cases where it
-has been given away.
-
-Arguments:
-  transport_name  to pass to the new process
-  hostname        ditto
-  hostaddress     ditto
-  id              the new message to process
-  socket_fd       the connected socket
-
-Returns:          FALSE if fork fails; TRUE otherwise
-*/
-
-BOOL
-transport_pass_socket(const uschar *transport_name, const uschar *hostname,
-  const uschar *hostaddress, uschar *id, int socket_fd
-#ifndef DISABLE_ESMTP_LIMITS
-  , unsigned peer_limit_mail, unsigned peer_limit_rcpt, unsigned peer_limit_rcptdom
-#endif
-  )
-{
-pid_t pid;
-int status;
-
-DEBUG(D_transport) debug_printf("transport_pass_socket entered\n");
-
-#ifndef DISABLE_ESMTP_LIMITS
-continue_limit_mail = peer_limit_mail;
-continue_limit_rcpt = peer_limit_rcpt;
-continue_limit_rcptdom = peer_limit_rcptdom;
-#endif
-
-if ((pid = exim_fork(US"continued-transport-interproc")) == 0)
-  {
-  /* Disconnect entirely from the parent process. If we are running in the
-  test harness, wait for a bit to allow the previous process time to finish,
-  write the log, etc., so that the output is always in the same order for
-  automatic comparison. */
-
-  if ((pid = exim_fork(US"continued-transport")) != 0)
-    _exit(EXIT_SUCCESS);
-  testharness_pause_ms(1000);
-
-  transport_do_pass_socket(transport_name, hostname, hostaddress,
-    id, socket_fd);
-  }
-
-/* If the process creation succeeded, wait for the first-level child, which
-immediately exits, leaving the second level process entirely disconnected from
-this one. */
-
-if (pid > 0)
-  {
-  int rc;
-  while ((rc = wait(&status)) != pid && (rc >= 0 || errno != ECHILD));
-  return TRUE;
-  }
-else
-  {
-  DEBUG(D_transport) debug_printf("transport_pass_socket failed to fork: %s\n",
-    strerror(errno));
-  return FALSE;
-  }
-}
-
-
 
 /* Enforce all args untainted, for consistency with a router-sourced pipe
 command, where (because the whole line is passed as one to the tpt) a
@@ -2201,7 +2231,7 @@ argv[argcount] = NULL;
 
 if (*s)
   {
-  uschar *msg = string_sprintf("Too many arguments in command \"%s\" in "
+  uschar *msg = string_sprintf("Too many arguments in command %q in "
     "%s", cmd, etext);
   if (addr)
     {
@@ -2255,7 +2285,7 @@ if (flags & TSUC_EXPAND_ARGS)
       if (argcount + address_count - 1 > max_args)
         {
         addr->transport_return = FAIL;
-        addr->message = string_sprintf("Too many arguments to command \"%s\" "
+        addr->message = string_sprintf("Too many arguments to command %q "
           "in %s", cmd, etext);
         return FALSE;
         }
@@ -2306,13 +2336,13 @@ if (flags & TSUC_EXPAND_ARGS)
       address_pipe_argv = store_get((address_pipe_max_args+1)*sizeof(uschar *), GET_UNTAINTED);
 
       /* +1 because addr->local_part[0] == '|' since af_force_command is set */
-      s = expand_cstring(addr->local_part + 1);
+      s = expand_string(addr->local_part + 1);
 
       if (!s || !*s)
         {
         addr->transport_return = FAIL;
-        addr->message = string_sprintf("Expansion of \"%s\" "
-           "from command \"%s\" in %s failed: %s",
+        addr->message = string_sprintf("Expansion of %q "
+           "from command %q in %s failed: %s",
            (addr->local_part + 1), cmd, etext, expand_string_message);
         return FALSE;
         }
@@ -2337,14 +2367,10 @@ if (flags & TSUC_EXPAND_ARGS)
       /* If *s != 0 we have run out of argument slots. */
       if (*s)
         {
-        uschar *msg = string_sprintf("Too many arguments in $address_pipe "
-          "\"%s\" in %s", addr->local_part + 1, etext);
-        if (addr)
-          {
-          addr->transport_return = FAIL;
-          addr->message = msg;
-          }
-        else *errptr = msg;
+        uschar * msg = string_sprintf("Too many arguments in $address_pipe "
+          "%q in %s", addr->local_part + 1, etext);
+	addr->transport_return = FAIL;
+	addr->message = msg;
         return FALSE;
         }
 
@@ -2356,7 +2382,7 @@ if (flags & TSUC_EXPAND_ARGS)
         {
         addr->transport_return = FAIL;
         addr->message = string_sprintf("Too many arguments to command "
-          "\"%s\" after expanding $address_pipe in %s", cmd, etext);
+          "%q after expanding $address_pipe in %s", cmd, etext);
         return FALSE;
         }
 
@@ -2384,9 +2410,9 @@ if (flags & TSUC_EXPAND_ARGS)
            address_pipe_argv[address_pipe_i];
            address_pipe_i++, argcount++)
 	{
-        uschar * s = address_pipe_argv[address_pipe_i];
-	if (arg_is_tainted(s, i, addr, etext, errptr)) return FALSE;
-        argv[i++] = s;
+        uschar * t = address_pipe_argv[address_pipe_i];
+	if (arg_is_tainted(t, i, addr, etext, errptr)) return FALSE;
+        argv[i++] = t;
 	}
 
       /* Subtract one since we replace $address_pipe */
@@ -2398,16 +2424,16 @@ if (flags & TSUC_EXPAND_ARGS)
 
     else
       {
-      const uschar *expanded_arg;
+      const uschar * expanded_arg;
       BOOL enable_dollar_recipients_g = f.enable_dollar_recipients;
       f.enable_dollar_recipients = allow_dollar_recipients;
-      expanded_arg = expand_cstring(argv[i]);
+      expanded_arg = expand_string(argv[i]);
       f.enable_dollar_recipients = enable_dollar_recipients_g;
 
       if (!expanded_arg)
         {
-        uschar *msg = string_sprintf("Expansion of \"%s\" "
-          "from command \"%s\" in %s failed: %s",
+        uschar *msg = string_sprintf("Expansion of %q "
+          "from command %q in %s failed: %s",
           argv[i], cmd, etext, expand_string_message);
         if (addr)
           {
