@@ -2,6 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -40,7 +41,7 @@ extern autoreply_transport_options_block autoreply_transport_option_defaults;
 
 /* The main and init entry points for the transport */
 
+extern void autoreply_transport_init(driver_instance *);
 extern BOOL autoreply_transport_entry(transport_instance *, address_item *);
-extern void autoreply_transport_init(transport_instance *);
 
 /* End of transports/autoreply.h */
